@@ -1,23 +1,26 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2014 Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 1998, 2023 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019 IBM Corporation. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- *     07/16/2009-2.0 Guy Pelletier 
- *       - 277039: JPA 2.0 Cache Usage Settings
- *     10/29/2010-2.2 Michael O'Brien 
- *       - 325167: Make reserved # bind parameter char generic to enable native SQL pass through
- *     06/30/2011-2.3.1 Guy Pelletier 
- *       - 341940: Add disable/enable allowing native queries    
- *     09/03/2015 - Will Dazey
- *       - 456067 : Added support for defining query timeout units
- ******************************************************************************/  
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     07/16/2009-2.0 Guy Pelletier
+//       - 277039: JPA 2.0 Cache Usage Settings
+//     10/29/2010-2.2 Michael O'Brien
+//       - 325167: Make reserved # bind parameter char generic to enable native SQL pass through
+//     06/30/2011-2.3.1 Guy Pelletier
+//       - 341940: Add disable/enable allowing native queries
+//     09/03/2015 - Will Dazey
+//       - 456067 : Added support for defining query timeout units
 package org.eclipse.persistence.config;
 
 import org.eclipse.persistence.annotations.BatchFetch;
@@ -27,15 +30,15 @@ import org.eclipse.persistence.annotations.BatchFetchType;
  * The class defines EclipseLink query hints.
  * These query hints allow a JPA Query to be customized or optimized beyond
  * what is available in the JPA specification.
- * 
+ *
  * <p>JPA Query Hint Usage:
- * 
+ *
  * <p><code>query.setHint(QueryHints.CACHE_USAGE, CacheUsage.CheckCacheOnly);</code>
- * <p>or 
+ * <p>or
  * <p><code>@QueryHint(name=QueryHints.CACHE_USAGE, value=CacheUsage.CheckCacheOnly)</code>
- * 
+ *
  * <p>Hint values are case-insensitive; "" could be used instead of default value.
- * 
+ *
  * @see HintValues
  * @see CacheUsage
  * @see PessimisticLock
@@ -61,28 +64,28 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setShouldBindAllParameters(boolean)
      */
     public static final String PARAMETER_DELIMITER = "eclipselink.jdbc.parameter-delimiter";
-    
+
     /**
      * "javax.persistence.cache.retrieveMode"
-     * <p>Configures the behavior when data is retrieved by the find methods and 
-     * by the execution of queries. The cache retrieve mode is ignored for the 
-     * refresh method, which always causes data to be retrieved from the 
+     * <p>Configures the behavior when data is retrieved by the find methods and
+     * by the execution of queries. The cache retrieve mode is ignored for the
+     * refresh method, which always causes data to be retrieved from the
      * database and not the cache.</p>
      * @see org.eclipse.persistence.queries.DatabaseQuery#dontMaintainCache()
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#shouldCheckDescriptorForCacheUsage()
      */
     public static final String CACHE_RETRIEVE_MODE = "javax.persistence.cache.retrieveMode";
-    
+
     /**
      * "javax.persistence.cache.storeMode"
-     * <p>Configures the behavior when data is read from the database and when 
+     * <p>Configures the behavior when data is read from the database and when
      * data is committed into the database.</p>
      * @see org.eclipse.persistence.queries.DatabaseQuery#dontMaintainCache()
      * @see org.eclipse.persistence.queries.DatabaseQuery#maintainCache()
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#refreshIdentityMapResult()
      */
     public static final String CACHE_STORE_MODE = "javax.persistence.cache.storeMode";
-    
+
     /**
      * "eclipselink.cache-usage"
      * <p>Configures the query to utilize the EclipseLink cache, by default the cache is not checked on queries before accessing the database.
@@ -94,7 +97,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setCacheUsage(int)
      */
     public static final String CACHE_USAGE = "eclipselink.cache-usage";
-    
+
     /**
      * "eclipselink.query-results-cache"
      * <p>Configures the query to use a results cache.
@@ -153,12 +156,12 @@ public class QueryHints {
     /**
      * "eclipselink.query-results-cache.invalidate-on-change"
      * <p>Configures if the query cache results should be invalidated if any object of any class used in the query if modified.
-     * 
+     *
      * Valid values are "true" and "false", true is the default.
      * @see org.eclipse.persistence.queries.QueryResultsCachePolicy#setInvalidateOnChange(boolean)
      */
     public static final String QUERY_RESULTS_CACHE_INVALIDATE = "eclipselink.query-results-cache.invalidate-on-change";
-    
+
     /**
      * "eclipselink.query-results-cache.type"
      * <p>Configures the cache type of the query's results cache.
@@ -168,7 +171,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.QueryResultsCachePolicy#setCacheType(java.lang.Class)
      */
     public static final String QUERY_RESULTS_CACHE_TYPE = "eclipselink.query-results-cache.type";
-    
+
     /**
      * "eclipselink.query-results-cache.expiry-time-of-day"
      * <p>Configures the time of day expiry time of the query's results cache.
@@ -178,7 +181,22 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.QueryResultsCachePolicy#setCacheInvalidationPolicy(org.eclipse.persistence.descriptors.invalidation.CacheInvalidationPolicy)
      */
     public static final String QUERY_RESULTS_CACHE_EXPIRY_TIME_OF_DAY = "eclipselink.query-results-cache.expiry-time-of-day";
-    
+
+    /**
+     * <p>
+     * This property control (enable/disable) query result cache validation in {@link org.eclipse.persistence.internal.sessions.UnitOfWorkImpl#internalExecuteQuery}
+     * </p>
+     * This can be used to help debugging an object identity problem. An object identity problem is when an managed/active entity in the cache references an entity not in managed state.
+     * This method will validate that objects in query results are in a correct state. As a result there are new log messages in the log.
+     * It's related with "read" queries like <code>em.find(...);</code> or JPQL queries like <code>SELECT e FROM Entity e</code>.
+     * It should be controlled at persistence unit level too by persistence unit property {@link org.eclipse.persistence.config.PersistenceUnitProperties#QUERY_RESULTS_CACHE_VALIDATION}
+     * <ul>
+     * <li>"<code>true</code>" - validate query result object tree and if content is not valid print diagnostic messages. In this case there should be negative impact to the performance.
+     * <li>"<code>false</code>" (DEFAULT) - don't validate and print any diagnostic messages
+     * </ul>
+     */
+    public static final String QUERY_RESULTS_CACHE_VALIDATION = "eclipselink.query-results-cache.validation";
+
     /**
      * "eclipselink.query.redirector"
      * <p>Used to provide a QueryRedirector to the executing query.
@@ -192,7 +210,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setRedirector(org.eclipse.persistence.queries.QueryRedirector)
      */
     public static final String QUERY_REDIRECTOR = "eclipselink.query.redirector";
-    
+
     /**
      * "eclipselink.partitioning"
      * <p>Used to provide a PartitioningPolicy instance or name to the query.
@@ -213,7 +231,7 @@ public class QueryHints {
      * @see QueryType
      */
     public static final String QUERY_TYPE = "eclipselink.query-type";
-    
+
     /**
      * "eclipselink.pessimistic-lock"
      * <p>Configures  the query to acquire a pessimistic lock (write-lock) on the resulting rows in the database.
@@ -221,19 +239,19 @@ public class QueryHints {
      * Pessimistic locking support and behavior may differ on difference database platforms.
      * @see PessimisticLock
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setLockMode(short)
-     * 
+     *
      * <B>Note:</B> As of JPA 2.0 there is a standard way to configure pessimistic locking.
      * @see javax.persistence.LockModeType
      * @see javax.persistence.EntityManager (find(), refresh(), lock())
      * @see javax.persistence.Query (setLockMode())
      */
     public static final String PESSIMISTIC_LOCK = "eclipselink.pessimistic-lock";
-    
+
     /**
      * "javax.persistence.lock.timeout"
-     * <p>Configures the WAIT timeout used in pessimistic locking, if the database 
-     * query exceeds the timeout the database will terminate the query and 
-     * return an exception. Valid values are Integer or Strings that can be 
+     * <p>Configures the WAIT timeout used in pessimistic locking, if the database
+     * query exceeds the timeout the database will terminate the query and
+     * return an exception. Valid values are Integer or Strings that can be
      * parsed to int values.
      * Some database platforms may not support lock timeouts, you may consider
      * setting a JDBC_TIMEOUT hint for these platforms.
@@ -241,7 +259,20 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setWaitTimeout(Integer)
      */
     public static final String PESSIMISTIC_LOCK_TIMEOUT = "javax.persistence.lock.timeout";
-    
+
+    /**
+     * "eclipselink.pessimistic.lock.timeout.unit"
+     * <p>Configures the pessimistic lock timeout unit value. Allows users more refinement.
+     * <b>Valid Values:</b>
+     * <ul>
+     * <li>"<code>java.util.concurrent.TimeUnit.MILLISECONDS</code>" (DEFAULT),
+     * <li>"<code>java.util.concurrent.TimeUnit.SECONDS</code>",
+     * <li>"<code>java.util.concurrent.TimeUnit.MINUTES</code>".
+     * </ul>
+     * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setWaitTimeoutUnit(TimeUnit)
+     */
+    public static final String PESSIMISTIC_LOCK_TIMEOUT_UNIT = "eclipselink.pessimistic.lock.timeout.unit";
+
     /**
      * "eclipselink.refresh"
      * <p>Configures the query to refresh the resulting objects in the cache and persistent context with the current state of the database.
@@ -254,7 +285,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setShouldRefreshIdentityMapResult(boolean)
      */
     public static final String REFRESH = "eclipselink.refresh";
-    
+
     /**
      * "eclipselink.batch"
      * <p>Configures the query to optimize the retrieval of the related objects,
@@ -266,7 +297,7 @@ public class QueryHints {
      *  'where clause' of the first SQL as part of the 'where clause' of the second.  When indirect attributes
      *  of the objects returned from the first query are referenced the second query is executed to return those attributes.
      *  If changes are made to data referenced in the 'where clause' after the first SQL results are returned but before the
-     *  second query executes then these attributes may not be returned by the second query. To prevent this reference these 
+     *  second query executes then these attributes may not be returned by the second query. To prevent this reference these
      *  attributes before changing data associated with the query."
      * @see BatchFetch
      * @see #BATCH_TYPE
@@ -298,7 +329,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setBatchFetchSize(int)
      */
     public static final String BATCH_SIZE = "eclipselink.batch.size";
-    
+
     /**
      * "eclipselink.join-fetch"
      * <p>Configures the query to optimize the retrieval of the related objects,
@@ -313,7 +344,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#addJoinedAttribute(String)
      */
     public static final String FETCH = "eclipselink.join-fetch";
-    
+
     /**
      * "eclipselink.left-join-fetch"
      * <p>Configures the query to optimize the retrieval of the related objects,
@@ -328,7 +359,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#addJoinedAttribute(String)
      */
     public static final String LEFT_FETCH = "eclipselink.left-join-fetch";
-    
+
     /**
      * "eclipselink.read-only"
      * <p>Configures the query to return shared (read-only) objects from the cache,
@@ -339,7 +370,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setIsReadOnly(boolean)
      */
     public static final String READ_ONLY = "eclipselink.read-only";
-    
+
     /**
      * "eclipselink.jdbc.timeout"
      * <p>Configures the JDBC timeout of the query execution, if the database query exceeds the timeout
@@ -348,7 +379,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setQueryTimeout(int)
      */
     public static final String JDBC_TIMEOUT = "eclipselink.jdbc.timeout";
-    
+
     /**
      * "javax.persistence.query.timeout" 
      * <p>Configures the default query timeout value per the JPA specification.
@@ -363,8 +394,8 @@ public class QueryHints {
      * <p>Configures the query timeout unit value. Allows users more refinement.
      * <b>Valid Values:</b>
      * <ul>
-     * <li>"<code>java.util.concurrent.TimeUnit.MILLISECONDS</code>",
-     * <li>"<code>java.util.concurrent.TimeUnit.SECONDS</code>" (DEFAULT),
+     * <li>"<code>java.util.concurrent.TimeUnit.MILLISECONDS</code>" (DEFAULT),
+     * <li>"<code>java.util.concurrent.TimeUnit.SECONDS</code>",
      * <li>"<code>java.util.concurrent.TimeUnit.MINUTES</code>".
      * </ul>
      * @see org.eclipse.persistence.queries.DatabaseQuery#setQueryTimeoutUnit(TimeUnit)
@@ -379,7 +410,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ReadQuery#setFetchSize(int)
      */
     public static final String JDBC_FETCH_SIZE = "eclipselink.jdbc.fetch-size";
-    
+
     /**
      * "eclipselink.jdbc.max-rows"
      * <p>Configures the JDBC max-rows, if the query returns more rows than the max-rows
@@ -390,7 +421,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ReadQuery#setMaxRows(int)
      */
     public static final String JDBC_MAX_ROWS = "eclipselink.jdbc.max-rows";
-    
+
     /**
      * "eclipselink.jdbc.first-result"
      * <p>Configures the query to skip the firstResult number of rows.
@@ -400,12 +431,12 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ReadQuery#setFirstResult(int)
      */
     public static final String JDBC_FIRST_RESULT = "eclipselink.jdbc.first-result";
-    
+
     /**
      * "eclipselink.result-collection-type"
      * <p>Configures the collection class implementation for the queries result.
      * The fully qualified class name must be used, without the .class.
-     * Valid values are a Class representing a collection type or a String representing the class' name 
+     * Valid values are a Class representing a collection type or a String representing the class' name
      * of the collection type.
      * If a Collection type that is not a List is used, getResultCollection() or getSingleResult() must be used instead of
      * getResultList().
@@ -414,7 +445,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ReadAllQuery#useCollectionClass(Class)
      */
     public static final String RESULT_COLLECTION_TYPE = "eclipselink.result-collection-type";
-    
+
     /**
      * "eclipselink.refresh.cascade"
      * <p>Defines if a refresh query should cascade the refresh to relationships.
@@ -425,7 +456,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setCascadePolicy(int)
      */
     public static final String REFRESH_CASCADE = "eclipselink.refresh.cascade";
-    
+
     /**
      * "eclipselink.maintain-cache"
      * <p>Configures the query to not use both the shared cache, and the transactional cache/persistence context.
@@ -444,7 +475,7 @@ public class QueryHints {
      * <p>Configures the query to not prepare its SQL.
      * By default queries generate their SQL the first time they are executed,
      * and avoid the cost of generating the SQL on subsequent executions.
-     * This can be used to generate the SQL on every execution if the 
+     * This can be used to generate the SQL on every execution if the
      * query requires usage of dynamic SQL, such as to handle null parameters (use IS NULL instead of = null).
      * This only effects the SQL generation, not parameter binding or statement caching.
      * Valid values are:  HintValues.FALSE, HintValues.TRUE,
@@ -469,7 +500,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setShouldCacheStatement(boolean)
      */
     public static final String CACHE_STATMENT = "eclipselink.jdbc.cache-statement";
-    
+
     /**
      * "eclipselink.flush"
      * <p>Configures if the query should trigger a flush of the persistence context before execution.
@@ -487,7 +518,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setFlushOnExecute(Boolean)
      */
     public static final String FLUSH = "eclipselink.flush";
-    
+
     /**
      * "eclipselink.sql.hint"
      * <p>
@@ -497,7 +528,7 @@ public class QueryHints {
      * A SQL hint can be used on certain database platforms to define how the query uses indexes
      * and other such low level usages. This should be the full hint string including the comment \
      * delimiters.
-     * 
+     *
      * @see org.eclipse.persistence.queries.DatabaseQuery#setHintString(String)
      */
     public static final String HINT = "eclipselink.sql.hint";
@@ -517,7 +548,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setIsNativeConnectionRequired(boolean)
      */
     public static final String NATIVE_CONNECTION = "eclipselink.jdbc.native-connection";
-    
+
     /**
      * "eclipselink.cursor"
      * <p>Configures the query to return a CursoredStream.
@@ -569,7 +600,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ReadAllQuery#useCursoredStream(int, int, org.eclipse.persistence.queries.ValueReadQuery)
      */
     public static final String CURSOR_SIZE = "eclipselink.cursor.size-sql";
-    
+
     /**
      * "eclipselink.cursor.scrollable"
      * <p>Configures the query to return a ScrollableCursor.
@@ -600,7 +631,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ScrollableCursorPolicy#setResultSetType(int)
      */
     public static final String RESULT_SET_TYPE = "eclipselink.cursor.scrollable.result-set-type";
-    
+
     /**
      * "eclipselink.cursor.scrollable.result-set-concurrency"
      * <p>This can be used on ScrollableCursor queries to set the JDBC ResultSet concurrency.
@@ -609,7 +640,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ScrollableCursorPolicy#setResultSetConcurrency(int)
      */
     public static final String RESULT_SET_CONCURRENCY = "eclipselink.cursor.scrollable.result-set-concurrency";
-    
+
     /**
      * "eclipselink.fetch-group"
      * <p>Configures the query to use the fetch group object.
@@ -629,7 +660,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setFetchGroup(org.eclipse.persistence.queries.FetchGroup)
      */
     public static final String FETCH_GROUP = "eclipselink.fetch-group";
-    
+
     /**
      * "eclipselink.fetch-group.name"
      * <p>Configures the query to use a named fetch group defined for the result class.
@@ -648,7 +679,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setFetchGroupName(String)
      */
     public static final String FETCH_GROUP_NAME = "eclipselink.fetch-group.name";
-    
+
     /**
      * "eclipselink.fetch-group.attribute"
      * <p>Configures the query to use a dynamic fetch group that includes a list of attributes.
@@ -668,7 +699,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setFetchGroup(org.eclipse.persistence.queries.FetchGroup)
      */
     public static final String FETCH_GROUP_ATTRIBUTE = "eclipselink.fetch-group.attribute";
-    
+
     /**
      * "eclipselink.fetch-group.load"
      * <p>Configures a dynamic fetch group to load (default) or not to load its attributes.
@@ -682,16 +713,16 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.FetchGroup#setShouldLoad(boolean)
      */
     public static final String FETCH_GROUP_LOAD = "eclipselink.fetch-group.load";
-    
+
     /**
      * "javax.persistence.fetchgraph"
-     * <p> Configures a jpa entity graph to be used as a fetch graph template.   
+     * <p> Configures a jpa entity graph to be used as a fetch graph template.
      * Those attributes included in the entity graph will have their values fetched and populated.
      * Attributes of the represented entity that are not present in the entity graph will be treated
      * as FetchType.LAZY.
      */
     public static final String JPA_FETCH_GRAPH = "javax.persistence.fetchgraph";
-    
+
     /**
      * "eclipselink.fetch-group.default"
      * <p>Configures the query not to use the default fetch group.
@@ -720,16 +751,16 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setLoadGroup(org.eclipse.persistence.queries.LoadGroup)
      */
     public static final String LOAD_GROUP = "eclipselink.load-group";
-    
+
     /**
      * "javax.persistence.loadgraph"
-     * <p> Configures a jpa entity graph to be used as a load graph template.   
+     * <p> Configures a jpa entity graph to be used as a load graph template.
      * Those attributes included in the entity graph will have their values fetched and populated.
      * Attributes of the represented entity that are not present in the entity graph will be loaded based
      * on their mapping based FetchType settings.
      */
     public static final String JPA_LOAD_GRAPH = "javax.persistence.loadgraph";
-    
+
     /**
      * "eclipselink.load-group.attribute"
      * <p>Configures the query to use load group that includes a list of attributes.
@@ -742,7 +773,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setLoadGroup
      */
     public static final String LOAD_GROUP_ATTRIBUTE = "eclipselink.load-group.attribute";
-    
+
     /**
      * "eclipselink.exclusive-connection"
      * <p>Configures the query to use the exclusive (transactional/write) connection.
@@ -759,7 +790,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setShouldUseExclusiveConnection(boolean)
      */
     public static final String EXCLUSIVE_CONNECTION = "eclipselink.exclusive-connection";
-    
+
     /**
      * "eclipselink.inheritance.outer-join"
      * <p>Configures the query to outer-join all subclasses.
@@ -774,10 +805,10 @@ public class QueryHints {
     public static final String INHERITANCE_OUTER_JOIN = "eclipselink.inheritance.outer-join";
 
     /**
-     * The <code>"eclipselink.jdbc.allow-native-sql-query"</code> property 
-     * specifies whether a single native SQL query should override a persistence 
-     * unit level setting (eclipselink.jdbc.allow-native-sql-queries). The PU 
-     * level flag is of particular importance within a multitenant to minimize 
+     * The <code>"eclipselink.jdbc.allow-native-sql-query"</code> property
+     * specifies whether a single native SQL query should override a persistence
+     * unit level setting (eclipselink.jdbc.allow-native-sql-queries). The PU
+     * level flag is of particular importance within a multitenant to minimize
      * the potential impact of revealing multitenant. However in some cases the
      * application may need to allow certain native SQL queries through.
      * <p>
@@ -786,11 +817,11 @@ public class QueryHints {
      * <li>"true" - allow native SQL (and override the persistence unit flag)
      * <li>"false" - (DEFAULT) do not allow native SQL (and respect the persistence unit flag if set.)
      * </ul>
-     * 
+     *
      * @see org.eclipse.persistence.queries.DatabaseQuery#setAllowNativeSQLQuery(Boolean) {
      */
     public static final String ALLOW_NATIVE_SQL_QUERY = "eclipselink.jdbc.allow-native-sql-query";
-    
+
     /**
      * "eclipselink.history.as-of"
      * <p>Configures the query to query the state of the object as-of a point in time.
@@ -801,7 +832,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setAsOfClause(org.eclipse.persistence.history.AsOfClause)
      */
     public static final String AS_OF = "eclipselink.history.as-of";
-    
+
     /**
      * "eclipselink.history.as-of.scn"
      * <p>Configures the query to query the state of the object as-of a database SCN (System Change Number).
@@ -828,7 +859,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DataReadQuery#setResultType
      */
     public static final String RESULT_TYPE = "eclipselink.result-type";
-    
+
     /**
      * "eclipselink.cache-usage.indirection-policy"
      * <p>This can be used on a query with a CACHE_USAGE hint to configure the behavior of in-memory
@@ -840,12 +871,12 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setInMemoryQueryIndirectionPolicyState(int)
      */
     public static final String INDIRECTION_POLICY = "eclipselink.cache-usage.indirection-policy";
-    
+
     /**
      * "javax.persistence.lock.scope"
      * <p> By default pessimistic lock applied to only the tables mapped to the object being locked.
      * <p> It could be extended to apply also to relation (join) tables (ManyToMany and OneToOne case),
-     * and CollectionTables (ElementCollection case). 
+     * and CollectionTables (ElementCollection case).
      * Valid values are defined in PessimisticLockScope.
      * @see javax.persistence.PessimisticLockScope
      * @see javax.persistence.LockModeType
@@ -859,7 +890,7 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.DatabaseQuery#setSessionName(String)
      */
     public static final String COMPOSITE_UNIT_MEMBER = "eclipselink.composite-unit.member";
-    
+
     /**
      * "eclipselink.jdbc.batch-writing"
      * <p>Configures if this modify query can be batched through batch writing.
@@ -875,7 +906,7 @@ public class QueryHints {
      * "eclipselink.result-set-access"
      * <p>Configures if this read object(s) query should keep result set until all result object(s) are built.
      * That allows to avoid getting potentially huge objects (LOBs) from ResultSet in case they would not be used
-     * because the object is found in the cache.   
+     * because the object is found in the cache.
      * Valid values are:  HintValues.PERSISTENCE_UNIT_DEFAULT, HintValues.TRUE, HintValues.FALSE,
      * "" could be used instead of default value HintValues.PERSISTENCE_UNIT_DEFAULT
      * @see org.eclipse.persistence.queries.ObjectLevelReadQuery#setIsResultSetAccessOptimizedQuery(boolean)
@@ -901,4 +932,13 @@ public class QueryHints {
      * @see org.eclipse.persistence.queries.ResultSetMappingQuery#shouldReturnNameValuePairs()
      */
     public static final String RETURN_NAME_VALUE_PAIRS = "eclipselink.query-return-name-value-pairs";
+
+    /**
+     * "eclipselink.inner-join-in-where-clause"
+     * <p>Changes the way that inner joins are printed in generated SQL for the database.
+     * With a value of true, inner joins are printed in the WHERE clause, if false, inner joins are printed in the FROM clause.
+     * This query hint should override global/session switch {@link org.eclipse.persistence.internal.databaseaccess.DatabasePlatform#setPrintInnerJoinInWhereClause(boolean)}
+     * @see org.eclipse.persistence.internal.databaseaccess.DatabasePlatform#setPrintInnerJoinInWhereClause(boolean)
+     */
+    public static final String INNER_JOIN_IN_WHERE_CLAUSE = "eclipselink.inner-join-in-where-clause";
 }
