@@ -1,17 +1,19 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2017 Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- *     12/14/2017-2.6.6 Tomas Kraus
- *       - 291546: Performance degradation due to usage of Vector in DescriptorEventManager
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     12/14/2017-3.0 Tomas Kraus
+//       - 291546: Performance degradation due to usage of Vector in DescriptorEventManager
 package org.eclipse.persistence.descriptors;
 
 import java.io.Serializable;
@@ -22,7 +24,6 @@ import java.security.PrivilegedActionException;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.CopyOnWriteArrayList;
-import java.util.concurrent.atomic.AtomicIntegerArray;
 import java.util.concurrent.atomic.AtomicReferenceArray;
 
 import org.eclipse.persistence.core.descriptors.CoreDescriptorEventManager;
@@ -36,9 +37,9 @@ import org.eclipse.persistence.sessions.SessionProfiler;
 
 /**
  * <p><b>Purpose</b>: The event manager allows for a descriptor to specify that
- * an object should be notified when a EclipseLink event occurs.  It also determines 
- * how the object will be notified. To specify an event a method name can be 
- * registered to be called on the object when the event occurs. Events can be 
+ * an object should be notified when a EclipseLink event occurs.  It also determines
+ * how the object will be notified. To specify an event a method name can be
+ * registered to be called on the object when the event occurs. Events can be
  * used to extend the EclipseLink reading and writing behavior.
  * <p>
  * These events include:
@@ -100,7 +101,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     // CR#2660080 was missing aboutToDelete
     public static final int AboutToDeleteEvent = 14;
-    
+
     // EJB 3.0 events
     public static final int PrePersistEvent = 15;
     public static final int PreRemoveEvent = 16;
@@ -130,7 +131,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * EJB 3.0 support for lifecycle callback events defined on an entity 
+     * EJB 3.0 support for lifecycle callback events defined on an entity
      * listener class.
      */
     public void addEntityListenerEventListener(DescriptorEventListener listener) {
@@ -139,7 +140,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * Listener objects can be registered with the event manager to be notified 
+     * Listener objects can be registered with the event manager to be notified
      * when an event occurs on any instance of the descriptor's class.
      */
     public void addListener(DescriptorEventListener listener) {
@@ -166,7 +167,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
     public void addEntityListenerHolder(SerializableDescriptorEventHolder holder) {
         this.getDescriptorEventHolders().add(holder);
     }
-    
+
     /**
      * INTERNAL:
      * Clone the manager and its private parts.
@@ -183,11 +184,12 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
             throw new AssertionError(exception);
         }
     }
+
     /**
      * INTERNAL:
      * This method was added to allow JPA project caching so that DescriptorEventListeners could be
      * serialized and re-added to the EventManager using a SerializableDescriptorEventHolder.
-     * @param classLoader 
+     * @param classLoader
      */
     public void processDescriptorEventHolders(AbstractSession session, ClassLoader classLoader) {
         if (this.descriptorEventHolders != null) {
@@ -199,7 +201,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * INTERNAL:
-     * EJB 3.0 support. Returns true if this event manager should exclude the 
+     * EJB 3.0 support. Returns true if this event manager should exclude the
      * invocation of the default listeners for this descriptor.
      */
     public boolean excludeDefaultListeners() {
@@ -208,19 +210,20 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * INTERNAL:
-     * EJB 3.0 support. Returns true is this event manager should exclude the 
-     * invocation of the listeners defined by the entity listener classes for 
+     * EJB 3.0 support. Returns true is this event manager should exclude the
+     * invocation of the listeners defined by the entity listener classes for
      * the superclasses of this descriptor.
      */
     public boolean excludeSuperclassListeners() {
         return excludeSuperclassListeners;
     }
-    
+
     /**
      * INTERNAL:
      * Execute the given selector with the event as argument.
      * @exception DescriptorException - the method cannot be found or executed
      */
+    @Override
     public void executeEvent(DescriptorEvent event) throws DescriptorException {
         try {
             event.getSession().startOperationProfile(SessionProfiler.DescriptorEvent);
@@ -271,8 +274,8 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
     }
 
     /**
-     * Find the method corresponding to the event selector. The method MUST take 
-     * DescriptorEvent as argument, Session is also supported as argument for 
+     * Find the method corresponding to the event selector. The method MUST take
+     * DescriptorEvent as argument, Session is also supported as argument for
      * backward compatibility.
      */
     protected Method findMethod(int selector) throws DescriptorException {
@@ -294,21 +297,21 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      * bug 251180 - Missing method org.eclipse.persistence.descriptors.DescriptorEventManager#setAboutToDeleteSelector
      */
     public String getAboutToDeleteSelector() {
-        return (String)getEventSelectors().get(AboutToDeleteEvent);
+        return getEventSelectors().get(AboutToDeleteEvent);
     }
 
     /**
      * INTERNAL:
      */
     public String getAboutToInsertSelector() {
-        return (String)getEventSelectors().get(AboutToInsertEvent);
+        return getEventSelectors().get(AboutToInsertEvent);
     }
 
     /**
      * INTERNAL:
      */
     public String getAboutToUpdateSelector() {
-        return (String)getEventSelectors().get(AboutToUpdateEvent);
+        return getEventSelectors().get(AboutToUpdateEvent);
     }
 
     /**
@@ -357,7 +360,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
     public DescriptorEventListener getEntityEventListener() {
         return entityEventListener;
     }
-    
+
     /**
      * INTERNAL:
      * EJB 3.0 support. Returns the entity listener event listeners.
@@ -403,7 +406,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is built
      */
     public String getPostBuildSelector() {
-        return (String)getEventSelectors().get(PostBuildEvent);
+        return getEventSelectors().get(PostBuildEvent);
     }
 
     /**
@@ -411,7 +414,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is cloned
      */
     public String getPostCloneSelector() {
-        return (String)getEventSelectors().get(PostCloneEvent);
+        return getEventSelectors().get(PostCloneEvent);
     }
 
     /**
@@ -419,7 +422,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is deleted
      */
     public String getPostDeleteSelector() {
-        return (String)getEventSelectors().get(PostDeleteEvent);
+        return getEventSelectors().get(PostDeleteEvent);
     }
 
     /**
@@ -427,7 +430,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is inserted
      */
     public String getPostInsertSelector() {
-        return (String)getEventSelectors().get(PostInsertEvent);
+        return getEventSelectors().get(PostInsertEvent);
     }
 
     /**
@@ -435,7 +438,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is merged
      */
     public String getPostMergeSelector() {
-        return (String)getEventSelectors().get(PostMergeEvent);
+        return getEventSelectors().get(PostMergeEvent);
     }
 
     /**
@@ -443,7 +446,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is refreshed
      */
     public String getPostRefreshSelector() {
-        return (String)getEventSelectors().get(PostRefreshEvent);
+        return getEventSelectors().get(PostRefreshEvent);
     }
 
     /**
@@ -451,7 +454,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is updated
      */
     public String getPostUpdateSelector() {
-        return (String)getEventSelectors().get(PostUpdateEvent);
+        return getEventSelectors().get(PostUpdateEvent);
     }
 
     /**
@@ -459,7 +462,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called after an object is written
      */
     public String getPostWriteSelector() {
-        return (String)getEventSelectors().get(PostWriteEvent);
+        return getEventSelectors().get(PostWriteEvent);
     }
 
     /**
@@ -467,7 +470,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called before the create operation is applied to an object
      */
     public String getPrePersistSelector() {
-        return (String)getEventSelectors().get(PrePersistEvent);
+        return getEventSelectors().get(PrePersistEvent);
     }
 
   /**
@@ -475,7 +478,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called before an object is deleted
      */
     public String getPreDeleteSelector() {
-        return (String)getEventSelectors().get(PreDeleteEvent);
+        return getEventSelectors().get(PreDeleteEvent);
     }
 
     /**
@@ -483,7 +486,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called before an object is inserted
      */
     public String getPreInsertSelector() {
-        return (String)getEventSelectors().get(PreInsertEvent);
+        return getEventSelectors().get(PreInsertEvent);
     }
 
     /**
@@ -491,7 +494,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called before the remove operation is applied to an object
      */
     public String getPreRemoveSelector() {
-        return (String)getEventSelectors().get(PreRemoveEvent);
+        return getEventSelectors().get(PreRemoveEvent);
     }
 
   /**
@@ -499,7 +502,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called before an object is updated
      */
     public String getPreUpdateSelector() {
-        return (String)getEventSelectors().get(PreUpdateEvent);
+        return getEventSelectors().get(PreUpdateEvent);
     }
 
     /**
@@ -507,7 +510,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      *  The name of the method called before an object is written
      */
     public String getPreWriteSelector() {
-        return (String)getEventSelectors().get(PreWriteEvent);
+        return getEventSelectors().get(PreWriteEvent);
     }
 
     /**
@@ -515,17 +518,18 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
      * Return if the event manager has any event listeners, or event methods.
      * If nothing is listening to event they can be avoided.
      */
+    @Override
     public boolean hasAnyEventListeners() {
-        // Check listeners in case of collection added to directly as occurs 
-        // for aggregates that have a clone of the event manager but not the 
+        // Check listeners in case of collection added to directly as occurs
+        // for aggregates that have a clone of the event manager but not the
         // listeners.
         return hasAnyEventListeners || hasAnyListeners();
-    }   
-    
+    }
+
     protected boolean hasAnyListeners() {
         return (eventListeners != null) && (!eventListeners.isEmpty());
     }
-    
+
     /**
      * INTERNAL:
      * This method will return true, if this event manager has default listeners
@@ -535,19 +539,19 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
     public boolean hasDefaultEventListeners() {
         return defaultEventListeners != null && ! defaultEventListeners.isEmpty() && ! excludeDefaultListeners;
     }
-    
-    /** 
+
+    /**
      * INTERNAL:
-     * EJB 3.0 support. Return true if this event manager has any entity event 
+     * EJB 3.0 support. Return true if this event manager has any entity event
      * listeners.
      */
     public boolean hasEntityEventListener() {
         return entityEventListener != null;
     }
-    
-    /** 
+
+    /**
      * INTERNAL:
-     * Internal event support.  Return true if this event manager has any internal 
+     * Internal event support.  Return true if this event manager has any internal
      * listener event listeners.
      */
     public boolean hasInternalEventListeners() {
@@ -556,7 +560,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * INTERNAL:
-     * EJB 3.0 support. Return true if this event manager has any entity 
+     * EJB 3.0 support. Return true if this event manager has any entity
      * listener event listeners.
      */
     public boolean hasEntityListenerEventListeners() {
@@ -574,7 +578,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
         if (hasEntityEventListener() || hasEntityListenerEventListeners() || hasDefaultEventListeners() || hasInternalEventListeners()) {
             setHasAnyEventListeners(true);
         }
-        
+
         // Initialize if events are required at all.
         if (hasAnyListeners() || DescriptorEventListener.class.isAssignableFrom(getDescriptor().getJavaClass())) {
             setHasAnyEventListeners(true);
@@ -593,7 +597,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
         if (getDescriptor().isChildDescriptor()) {
             DescriptorEventManager parentEventManager = getDescriptor().getInheritancePolicy().getParentDescriptor().getEventManager();
             if (parentEventManager.hasAnyEventListeners()) {
-                setHasAnyEventListeners(true);                
+                setHasAnyEventListeners(true);
             }
 
             for (int index = 0; index < NumberOfEvents; index++) {
@@ -605,10 +609,10 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
             }
         }
     }
-    
+
     /**
      * INTERNAL:
-     * EJB 3.0 support. Builds our chains of descriptor event managers that will 
+     * EJB 3.0 support. Builds our chains of descriptor event managers that will
      * need to be notified. The chains are cache so we only need to build them
      * once.
      */
@@ -619,33 +623,33 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
         if (hasEntityEventListener()) {
             entityEventManagers.add(this);
         }
-        
+
         if (hasEntityListenerEventListeners()) {
             entityListenerEventManagers.add(this);
         }
-        
-        ClassDescriptor currentDescriptor = getDescriptor();     
+
+        ClassDescriptor currentDescriptor = getDescriptor();
         boolean excludeEntityListeners = excludeSuperclassListeners();
-            
+
         while (currentDescriptor.isChildDescriptor()) {
             currentDescriptor = currentDescriptor.getInheritancePolicy().getParentDescriptor();
-            
+
             DescriptorEventManager eventManager = currentDescriptor.getEventManager();
-            
+
             if (eventManager.hasEntityEventListener()) {
                 entityEventManagers.add(eventManager);
             }
-            
+
             if (eventManager.hasEntityListenerEventListeners()) {
                 if (!excludeEntityListeners) {
                     entityListenerEventManagers.add(eventManager);
                 }
             }
-            
+
             excludeEntityListeners = eventManager.excludeSuperclassListeners();
         }
     }
-    
+
     /**
      * INTERNAL:
      * Notify the EJB 3.0 event listeners.
@@ -686,7 +690,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
             }
         }
     }
-    
+
     /**
      * INTERNAL:
      * Big ugly case statement to notify listeners.
@@ -789,10 +793,10 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called when an object's row it about to 
+     * A method can be registered to be called when an object's row it about to
      * be inserted. This uses the optional event argument of the DatabaseRow.
-     * This is different from pre/postInsert because it occurs after the row has 
-     * already been built. This event can be used to modify the row before 
+     * This is different from pre/postInsert because it occurs after the row has
+     * already been built. This event can be used to modify the row before
      * insert, such as adding a user inserted by.
      */
     //bug 251180: Missing method org.eclipse.persistence.descriptors.DescriptorEventManager#setAboutToDeleteSelector
@@ -802,10 +806,10 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called when an object's row it about to 
+     * A method can be registered to be called when an object's row it about to
      * be inserted. This uses the optional event argument of the DatabaseRow.
-     * This is different from pre/postInsert because it occurs after the row has 
-     * already been built. This event can be used to modify the row before 
+     * This is different from pre/postInsert because it occurs after the row has
+     * already been built. This event can be used to modify the row before
      * insert, such as adding a user inserted by.
      */
     public void setAboutToInsertSelector(String aboutToInsertSelector) {
@@ -814,11 +818,11 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called when an object's row it about to 
+     * A method can be registered to be called when an object's row it about to
      * be updated. This uses the optional event argument of the DatabaseRow.
-     * This is different from pre/postUpdate because it occurs after the row has 
-     * already been built, and it ONLY called if the update is required (changed 
-     * within a unit of work), as the other occur ALWAYS. This event can be used 
+     * This is different from pre/postUpdate because it occurs after the row has
+     * already been built, and it ONLY called if the update is required (changed
+     * within a unit of work), as the other occur ALWAYS. This event can be used
      * to modify the row before insert, such as adding a user inserted by.
      */
     public void setAboutToUpdateSelector(String aboutToUpdateSelector) {
@@ -832,7 +836,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
     public void setDescriptor(ClassDescriptor descriptor) {
         this.descriptor = descriptor;
     }
-    
+
     /**
      * PUBLIC:
      * EJB 3.0 support for lifecycle callback events defined on an entity class.
@@ -860,7 +864,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
     /**
      * INTERNAL:
      * EJB 3.0 support. Default listeners apply to all entities in a persistence
-     * unit. Set this flag to true to exclude the invocation of the default 
+     * unit. Set this flag to true to exclude the invocation of the default
      * listeners for this descriptor.
      */
     public void setExcludeDefaultListeners(boolean excludeDefaultListeners) {
@@ -869,16 +873,16 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * INTERNAL:
-     * EJB 3.0 support. If multiple entity classes in an inheritance hierarchy 
-     * define entity listeners, the listeners defined for a superclass are 
-     * invoked before the listeners defined for its subclasses. Set this flag 
-     * to true to exclude the invocation of the listeners defined by the entity 
+     * EJB 3.0 support. If multiple entity classes in an inheritance hierarchy
+     * define entity listeners, the listeners defined for a superclass are
+     * invoked before the listeners defined for its subclasses. Set this flag
+     * to true to exclude the invocation of the listeners defined by the entity
      * listener classes for the superclasses of this descriptor.
      */
     public void setExcludeSuperclassListeners(boolean excludeSuperclassListeners) {
         this.excludeSuperclassListeners = excludeSuperclassListeners;
     }
-    
+
     /**
      * INTERNAL:
      * Set if the event manager has any event listeners, or event methods.
@@ -890,10 +894,10 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
-     * built from the database. This uses the optional event argument for the 
-     * DatabaseRow. This event can be used to correctly initialize an object's 
-     * non-persistent attributes or to perform complex optimizations or 
+     * A method can be registered to be called on a object that has just been
+     * built from the database. This uses the optional event argument for the
+     * DatabaseRow. This event can be used to correctly initialize an object's
+     * non-persistent attributes or to perform complex optimizations or
      * mappings. This event is called whenever an object is built.
      */
     public void setPostBuildSelector(String postBuildSelector) {
@@ -902,9 +906,9 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
-     * cloned into a unit of work. This uses the optional event argument for the 
-     * original object (the source object is the clone). This event can be used 
+     * A method can be registered to be called on a object that has just been
+     * cloned into a unit of work. This uses the optional event argument for the
+     * original object (the source object is the clone). This event can be used
      * to correctly initialize an object's non-persistent attributes.
      */
     public void setPostCloneSelector(String postCloneSelector) {
@@ -913,8 +917,8 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
-     * deleted from the database. This event can notify/remove any dependents 
+     * A method can be registered to be called on a object that has just been
+     * deleted from the database. This event can notify/remove any dependents
      * on the object.
      */
     public void setPostDeleteSelector(String postDeleteSelector) {
@@ -923,9 +927,9 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
-     * inserted into the database. This event can be used to notify any 
-     * dependent on the object, or to update information not accessible until 
+     * A method can be registered to be called on a object that has just been
+     * inserted into the database. This event can be used to notify any
+     * dependent on the object, or to update information not accessible until
      * the object has been inserted.
      */
     public void setPostInsertSelector(String postInsertSelector) {
@@ -934,10 +938,10 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
-     * merge from a unit of work. This uses the optional event argument of the 
-     * original object which is the object being merged from, the source object 
-     * is the object being merged into. This event can be used to correctly 
+     * A method can be registered to be called on a object that has just been
+     * merge from a unit of work. This uses the optional event argument of the
+     * original object which is the object being merged from, the source object
+     * is the object being merged into. This event can be used to correctly
      * initialize an object's non-persistent attributes.
      */
     public void setPostMergeSelector(String postMergeSelector) {
@@ -946,10 +950,10 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
-     * refreshed from the database. This uses the optional event argument of 
-     * the DatabaseRow. This event can be used to correctly initialize an 
-     * object's non-persistent attributes or to perform complex optimizations or 
+     * A method can be registered to be called on a object that has just been
+     * refreshed from the database. This uses the optional event argument of
+     * the DatabaseRow. This event can be used to correctly initialize an
+     * object's non-persistent attributes or to perform complex optimizations or
      * mappings. This event is only called on refreshes of existing objects.
      */
     public void setPostRefreshSelector(String postRefreshSelector) {
@@ -958,7 +962,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
+     * A method can be registered to be called on a object that has just been
      * updated into the database.
      */
     public void setPostUpdateSelector(String postUpdateSelector) {
@@ -967,12 +971,12 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that has just been 
-     * written to the database. This event is raised on any registered object 
-     * in a unit of work, even if it has not changed, refer to the 
-     * "aboutToUpdate" selector if it is required for the event to be raised 
-     * only when the object has been changed. This will be called on all inserts 
-     * and updates, after the "postInsert/Update" event has been raised. This 
+     * A method can be registered to be called on a object that has just been
+     * written to the database. This event is raised on any registered object
+     * in a unit of work, even if it has not changed, refer to the
+     * "aboutToUpdate" selector if it is required for the event to be raised
+     * only when the object has been changed. This will be called on all inserts
+     * and updates, after the "postInsert/Update" event has been raised. This
      * event can be used to notify any dependent on the object.
      */
     public void setPostWriteSelector(String postWriteSelector) {
@@ -981,8 +985,8 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that is going to be 
-     * deleted from the database. This event can notify/remove any dependents 
+     * A method can be registered to be called on a object that is going to be
+     * deleted from the database. This event can notify/remove any dependents
      * on the object.
      */
     public void setPreDeleteSelector(String preDeleteSelector) {
@@ -991,9 +995,9 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that is going to be 
-     * inserted into the database. This event can be used to notify any 
-     * dependent on the object or acquire the object's id through a custom 
+     * A method can be registered to be called on a object that is going to be
+     * inserted into the database. This event can be used to notify any
+     * dependent on the object or acquire the object's id through a custom
      * mechanism.
      */
     public void setPreInsertSelector(String preInsertSelector) {
@@ -1002,7 +1006,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object when that object has 
+     * A method can be registered to be called on a object when that object has
      * the create operation applied to it.
      */
     public void setPrePersistSelector(String prePersistSelector) {
@@ -1011,7 +1015,7 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object when that object has 
+     * A method can be registered to be called on a object when that object has
      * the remove operation applied to it.
      */
     public void setPreRemoveSelector(String preRemoveSelector) {
@@ -1020,11 +1024,11 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that is going to be 
-     * updated into the database. This event is raised on any registered object 
-     * in a unit of work, even if it has not changed, refer to the 
-     * "aboutToUpdate" selector if it is required for the event to be raised 
-     * only when the object has been changed. This event can be used to notify 
+     * A method can be registered to be called on a object that is going to be
+     * updated into the database. This event is raised on any registered object
+     * in a unit of work, even if it has not changed, refer to the
+     * "aboutToUpdate" selector if it is required for the event to be raised
+     * only when the object has been changed. This event can be used to notify
      * any dependent on the object.
      */
     public void setPreUpdateSelector(String preUpdateSelector) {
@@ -1033,12 +1037,12 @@ public class DescriptorEventManager extends CoreDescriptorEventManager<Descripto
 
     /**
      * PUBLIC:
-     * A method can be registered to be called on a object that is going to be 
-     * written to the database. This event is raised on any registered object 
-     * in a unit of work, even if it has not changed, refer to the 
-     * "aboutToUpdate" selector if it is required for the event to be raised 
-     * only when the object has been changed. This will be called on all inserts 
-     * and updates, before the "preInsert/Update" event has been raised. This 
+     * A method can be registered to be called on a object that is going to be
+     * written to the database. This event is raised on any registered object
+     * in a unit of work, even if it has not changed, refer to the
+     * "aboutToUpdate" selector if it is required for the event to be raised
+     * only when the object has been changed. This will be called on all inserts
+     * and updates, before the "preInsert/Update" event has been raised. This
      * event can be used to notify any dependent on the object.
      */
     public void setPreWriteSelector(String preWriteSelector) {
