@@ -1,17 +1,22 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2015 Oracle, IBM Corporation and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 1998, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2020 IBM Corporation and/or its affiliates. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- *     09/30/2014-2.6.0 Dalia Abo Sheasha
- *       - 445546: NullPointerException thrown when an Array of Bytes contains null values
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     09/30/2014-2.6.0 Dalia Abo Sheasha
+//       - 445546: NullPointerException thrown when an Array of Bytes contains null values
+//     05/11/2020-2.7.0 Jody Grassel
+//       - 538296: Wrong month is returned if OffsetDateTime is used in JPA 2.2 code
 package org.eclipse.persistence.internal.helper;
 
 import java.math.*;
@@ -20,13 +25,23 @@ import java.util.*;
 import java.io.*;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
-import java.sql.*;
+import java.sql.Blob;
+import java.sql.Clob;
+import java.sql.SQLException;
+import java.sql.Time;
+import java.sql.Timestamp;
+import java.time.LocalDateTime;
+import java.time.ZoneId;
+import java.time.ZoneOffset;
 
+import org.eclipse.persistence.config.SystemProperties;
 import org.eclipse.persistence.exceptions.*;
 import org.eclipse.persistence.internal.core.helper.CoreConversionManager;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
 import org.eclipse.persistence.internal.security.PrivilegedGetClassLoaderForClass;
 import org.eclipse.persistence.internal.security.PrivilegedGetContextClassLoader;
+import org.eclipse.persistence.logging.AbstractSessionLog;
+import org.eclipse.persistence.logging.SessionLog;
 
 /**
  * <p>
@@ -41,6 +56,58 @@ import org.eclipse.persistence.internal.security.PrivilegedGetContextClassLoader
  */
 public class ConversionManager extends CoreConversionManager implements Serializable, Cloneable {
     protected Map defaultNullValues;
+    private static ZoneId defaultZoneOffset = null;
+    
+    private static ZoneId getDefaultZoneOffset() {
+        if (defaultZoneOffset == null) {
+            ZoneId tzoneid = null;
+            String tzone = null;
+            try {
+                tzone = PrivilegedAccessHelper.getSystemProperty(SystemProperties.CONVERSION_USE_TIMEZONE);
+                if (tzone != null) {
+                    try {
+                        tzoneid = java.time.ZoneId.of(tzone);
+                    } catch (Throwable t) {
+                        // If an invalid time zone id is supplied, then fall back to checking for checking for using
+                        // either UTC or the system's default time zone.
+                        if (AbstractSessionLog.getLog().shouldLog(SessionLog.WARNING)) {
+                            AbstractSessionLog.getLog().log(SessionLog.WARNING, "invalid_tzone", 
+                                    SystemProperties.CONVERSION_USE_TIMEZONE, tzone);
+                        }
+                    }
+                } 
+            } catch (Exception e) {
+                // Error occurred attempting to access this system property.  Fall back to the next property.
+            }
+            
+            String propVal = null;
+            try {
+                if (tzoneid == null) {
+                    propVal = PrivilegedAccessHelper.getSystemProperty(SystemProperties.CONVERSION_USE_DEFAULT_TIMEZONE, "false");
+                    if (Boolean.parseBoolean(propVal)) {
+                        tzoneid = java.time.ZoneId.systemDefault();
+                    } else {
+                        tzoneid = ZoneOffset.UTC;
+                    }
+                }
+            } catch (Exception e) {
+                // Error occurred attempting to access this system property.  Fall back to UTC.
+                tzoneid = ZoneOffset.UTC;
+                if (AbstractSessionLog.getLog().shouldLog(SessionLog.WARNING)) {
+                    AbstractSessionLog.getLog().log(SessionLog.WARNING, "invalid_default_tzone", 
+                            SystemProperties.CONVERSION_USE_DEFAULT_TIMEZONE, propVal);
+                }
+            }           
+            
+            defaultZoneOffset = tzoneid;
+            
+            if (AbstractSessionLog.getLog().shouldLog(SessionLog.FINER)) {
+                AbstractSessionLog.getLog().log(SessionLog.FINER, "using_conversion_tzone", defaultZoneOffset);
+            }
+        }
+        
+        return defaultZoneOffset;        
+    }
 
     /**
      * This flag is here if the Conversion Manager should use the class loader on the
@@ -110,6 +177,16 @@ public class ConversionManager extends CoreConversionManager implements Serializ
                 return convertObjectToTime(sourceObject);
             } else if (javaClass == ClassConstants.TIMESTAMP) {
                 return convertObjectToTimestamp(sourceObject);
+            } else if (javaClass == ClassConstants.TIME_LDATE) {
+                return convertObjectToLocalDate(sourceObject);
+            } else if (javaClass == ClassConstants.TIME_LDATETIME) {
+                return convertObjectToLocalDateTime(sourceObject);
+            } else if (javaClass == ClassConstants.TIME_LTIME) {
+                return convertObjectToLocalTime(sourceObject);
+            } else if (javaClass == ClassConstants.TIME_ODATETIME) {
+                return convertObjectToOffsetDateTime(sourceObject);
+            } else if (javaClass == ClassConstants.TIME_OTIME) {
+                return convertObjectToOffsetTime(sourceObject);
             } else if ((javaClass == ClassConstants.CALENDAR) || (javaClass == ClassConstants.GREGORIAN_CALENDAR)) {
                 return convertObjectToCalendar(sourceObject);
             } else if ((javaClass == ClassConstants.CHAR) || (javaClass == ClassConstants.PCHAR && !(sourceObject instanceof Character))) {
@@ -120,7 +197,7 @@ public class ConversionManager extends CoreConversionManager implements Serializ
                 return convertObjectToDouble(sourceObject);
             } else if ((javaClass == ClassConstants.FLOAT) || (javaClass == ClassConstants.PFLOAT && !(sourceObject instanceof Float))) {
                 return convertObjectToFloat(sourceObject);
-        	} else if ((javaClass == ClassConstants.LONG) || (javaClass == ClassConstants.PLONG && !(sourceObject instanceof Long))) {
+            } else if ((javaClass == ClassConstants.LONG) || (javaClass == ClassConstants.PLONG && !(sourceObject instanceof Long))) {
                 return convertObjectToLong(sourceObject);
             } else if ((javaClass == ClassConstants.SHORT) || (javaClass == ClassConstants.PSHORT && !(sourceObject instanceof Short))) {
                 return convertObjectToShort(sourceObject);
@@ -152,15 +229,15 @@ public class ConversionManager extends CoreConversionManager implements Serializ
         } catch (Exception e) {
             throw ConversionException.couldNotBeConverted(sourceObject, javaClass, e);
         }
-        
+
         // Check if object is instance of the real class for the primitive class.
-        if ((((javaClass == ClassConstants.PBOOLEAN) && (sourceObject instanceof Boolean)  ) || 
-            ((javaClass == ClassConstants.PLONG) && (sourceObject instanceof Long) ) || 
-            ((javaClass == ClassConstants.PINT) && (sourceObject instanceof Integer)  ) || 
-            ((javaClass == ClassConstants.PFLOAT) && (sourceObject instanceof Float)) || 
-            ((javaClass == ClassConstants.PDOUBLE) &&  (sourceObject instanceof Double) ) || 
-            ((javaClass == ClassConstants.PBYTE) &&  (sourceObject instanceof Byte)) || 
-            ((javaClass == ClassConstants.PCHAR) &&  (sourceObject instanceof Character)) || 
+        if ((((javaClass == ClassConstants.PBOOLEAN) && (sourceObject instanceof Boolean)  ) ||
+            ((javaClass == ClassConstants.PLONG) && (sourceObject instanceof Long) ) ||
+            ((javaClass == ClassConstants.PINT) && (sourceObject instanceof Integer)  ) ||
+            ((javaClass == ClassConstants.PFLOAT) && (sourceObject instanceof Float)) ||
+            ((javaClass == ClassConstants.PDOUBLE) &&  (sourceObject instanceof Double) ) ||
+            ((javaClass == ClassConstants.PBYTE) &&  (sourceObject instanceof Byte)) ||
+            ((javaClass == ClassConstants.PCHAR) &&  (sourceObject instanceof Character)) ||
             ((javaClass == ClassConstants.PSHORT) &&  (sourceObject instanceof Short)))) {
             return sourceObject;
         }
@@ -189,9 +266,9 @@ public class ConversionManager extends CoreConversionManager implements Serializ
             } else if (sourceObject instanceof BigInteger) {
                 bigDecimal = new BigDecimal((BigInteger)sourceObject);
             } else if (sourceObject instanceof Number) {
-                // Doubles do not maintain scale, because of this it is 
+                // Doubles do not maintain scale, because of this it is
                 // impossible to distinguish between 1 and 1.0.  In order to
-                // maintain backwards compatibility both 1 and 1.0 will be 
+                // maintain backwards compatibility both 1 and 1.0 will be
                 // treated as BigDecimal(1).
                 String numberString = String.valueOf(sourceObject);
                 if(numberString.endsWith(".0") || numberString.contains(".0E+")) {
@@ -721,6 +798,10 @@ public class ConversionManager extends CoreConversionManager implements Serializ
             return Helper.timeFromCalendar((Calendar)sourceObject);
         } else if (sourceObject instanceof Long) {
             time = Helper.timeFromLong((Long)sourceObject);
+        } else if (sourceObject.getClass() == ClassConstants.TIME_LTIME) {
+            time = java.sql.Time.valueOf((java.time.LocalTime) sourceObject);
+        } else if (sourceObject.getClass() == ClassConstants.TIME_OTIME) {
+            time = java.sql.Time.valueOf(((java.time.OffsetTime) sourceObject).toLocalTime());
         } else {
             throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.TIME);
         }
@@ -730,7 +811,7 @@ public class ConversionManager extends CoreConversionManager implements Serializ
     /**
      * INTERNAL:
      * Build a valid instance of java.sql.Timestamp from the given source object.
-     * @param sourceObject    Valid obejct of class java.sql.Timestamp, String, java.util.Date, or Long
+     * @param sourceObject    Valid object of class java.sql.Timestamp, String, java.util.Date, or Long
      */
     protected java.sql.Timestamp convertObjectToTimestamp(Object sourceObject) throws ConversionException {
         java.sql.Timestamp timestamp = null;
@@ -747,12 +828,221 @@ public class ConversionManager extends CoreConversionManager implements Serializ
             return Helper.timestampFromCalendar((Calendar)sourceObject);
         } else if (sourceObject instanceof Long) {
             timestamp = Helper.timestampFromLong((Long)sourceObject);
+        } else if (sourceObject instanceof LocalDateTime) {
+            timestamp = Timestamp.valueOf((LocalDateTime) sourceObject);
         } else {
             throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.TIMESTAMP);
         }
         return timestamp;
     }
 
+    /**
+     * INTERNAL: Build a valid instance of java.time.LocalDate from the given
+     * source object.
+     *
+     * @param sourceObject
+     *            Valid object of class java.sql.Timestamp, String,
+     *            java.util.Date, or Long
+     */
+    protected java.time.LocalDate convertObjectToLocalDate(Object sourceObject) throws ConversionException {
+        java.time.LocalDate localDate = null;
+
+        if (sourceObject instanceof java.time.LocalDate) {
+            return (java.time.LocalDate) sourceObject;
+        }
+
+        if (sourceObject instanceof String) {
+            localDate = java.time.LocalDate.parse(((String) sourceObject).replace(' ', 'T'), Helper.getDefaultDateTimeFormatter());
+        } else if (sourceObject instanceof java.sql.Date) {
+            localDate = ((java.sql.Date) sourceObject).toLocalDate();
+        } else if (sourceObject instanceof java.sql.Timestamp) {
+            localDate = ((java.sql.Timestamp) sourceObject).toLocalDateTime().toLocalDate();
+        } else if (sourceObject instanceof java.util.Date) {
+            // handles sql.Time too
+            localDate = ((java.util.Date) sourceObject).toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
+        } else if (sourceObject instanceof Calendar) {
+            Calendar cal = (Calendar) sourceObject;
+            localDate = java.time.LocalDate.of(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH) + 1, cal.get(Calendar.DAY_OF_MONTH));
+        } else if (sourceObject instanceof Long) {
+            localDate = java.time.LocalDate.ofEpochDay((Long) sourceObject);
+        } else {
+            throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.TIME_LDATE);
+        }
+
+        return localDate;
+    }
+
+    /**
+     * INTERNAL: Build a valid instance of java.time.LocalTime from the given
+     * source object.
+     *
+     * @param sourceObject
+     *            Valid object of class java.sql.Timestamp, String,
+     *            java.util.Date, or Long
+     */
+    protected java.time.LocalTime convertObjectToLocalTime(Object sourceObject) throws ConversionException {
+        java.time.LocalTime localTime = null;
+
+        if (sourceObject instanceof java.time.LocalTime) {
+            return (java.time.LocalTime) sourceObject;
+        }
+
+        if (sourceObject instanceof String) {
+            localTime = java.time.LocalTime.parse(((String) sourceObject).replace(' ', 'T'), Helper.getDefaultDateTimeFormatter());
+        } else if (sourceObject instanceof java.sql.Timestamp) {
+            localTime = ((java.sql.Timestamp) sourceObject).toLocalDateTime().toLocalTime();
+        } else if (sourceObject instanceof java.sql.Time) {
+            localTime = ((java.sql.Time) sourceObject).toLocalTime();
+        } else if (sourceObject instanceof java.util.Date) {
+            // handles sql.Date
+            Calendar cal = Helper.allocateCalendar();
+            cal.setTime((java.util.Date) sourceObject);
+            localTime = java.time.LocalTime.of(
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND));
+            Helper.releaseCalendar(cal);
+        } else if (sourceObject instanceof Calendar) {
+            Calendar cal = (Calendar) sourceObject;
+            localTime = java.time.LocalTime.of(
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND));
+        } else if (sourceObject instanceof Long) {
+            localTime = java.time.LocalTime.ofSecondOfDay((Long) sourceObject);
+        } else {
+            throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.TIME_LTIME);
+        }
+
+        return localTime;
+    }
+
+    /**
+     * INTERNAL: Build a valid instance of java.time.LocalDateTime from the given
+     * source object.
+     *
+     * @param sourceObject
+     *            Valid object of class java.sql.Timestamp, String,
+     *            java.util.Date, or Long
+     */
+    protected java.time.LocalDateTime convertObjectToLocalDateTime(Object sourceObject) throws ConversionException {
+        java.time.LocalDateTime localDateTime = null;
+
+        if (sourceObject instanceof java.time.LocalDateTime) {
+            return (java.time.LocalDateTime) sourceObject;
+        }
+
+        if (sourceObject instanceof String) {
+            localDateTime = java.time.LocalDateTime.parse(((String) sourceObject).replace(' ', 'T'), Helper.getDefaultDateTimeFormatter());
+        } else if (sourceObject instanceof java.sql.Timestamp) {
+            localDateTime = ((java.sql.Timestamp) sourceObject).toLocalDateTime();
+        } else if (sourceObject instanceof java.util.Date) {
+            // handles sql.Time, sql.Date
+            Calendar cal = Helper.allocateCalendar();
+            cal.setTime((java.util.Date) sourceObject);
+            localDateTime = java.time.LocalDateTime.of(
+                    cal.get(Calendar.YEAR), cal.get(Calendar.MONTH) + 1, cal.get(Calendar.DAY_OF_MONTH),
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND));
+            Helper.releaseCalendar(cal);
+        } else if (sourceObject instanceof Calendar) {
+            Calendar cal = (Calendar) sourceObject;
+            localDateTime = java.time.LocalDateTime.of(
+                    cal.get(Calendar.YEAR), cal.get(Calendar.MONTH) + 1, cal.get(Calendar.DAY_OF_MONTH),
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND));
+        } else if (sourceObject instanceof Long) {
+            localDateTime = java.time.LocalDateTime.ofInstant(
+                    java.time.Instant.ofEpochSecond((Long) sourceObject), getDefaultZoneOffset());
+        } else {
+            throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.TIME_LDATETIME);
+        }
+
+        return localDateTime;
+    }
+
+    /**
+     * INTERNAL: Build a valid instance of java.time.OffsetDateTime from the given
+     * source object.
+     *
+     * @param sourceObject
+     *            Valid object of class java.sql.Timestamp, String,
+     *            java.util.Date, or Long
+     */
+    protected java.time.OffsetDateTime convertObjectToOffsetDateTime(Object sourceObject) throws ConversionException {
+        java.time.OffsetDateTime offsetDateTime = null;
+
+        if (sourceObject instanceof java.time.OffsetDateTime) {
+            return (java.time.OffsetDateTime) sourceObject;
+        }
+
+        if (sourceObject instanceof String) {
+            java.time.LocalDateTime ldt = java.time.LocalDateTime.parse(((String) sourceObject).replace(' ', 'T'), Helper.getDefaultDateTimeFormatter());
+            return ldt.atZone(java.time.ZoneId.systemDefault()).toOffsetDateTime();
+        } else if (sourceObject instanceof java.util.Date) {
+            // handles sql.Time, sql.Date, sql.Timestamp
+            Calendar cal = Helper.allocateCalendar();
+            cal.setTime((java.util.Date) sourceObject);
+            offsetDateTime = java.time.OffsetDateTime.of(
+                    cal.get(Calendar.YEAR), cal.get(Calendar.MONTH) + 1, cal.get(Calendar.DAY_OF_MONTH),
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND) * 1000000,
+                    java.time.ZoneOffset.ofTotalSeconds((cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET)) / 1000));
+            Helper.releaseCalendar(cal);
+        } else if (sourceObject instanceof Calendar) {
+            Calendar cal = (Calendar) sourceObject;
+            offsetDateTime = java.time.OffsetDateTime.of(
+                    cal.get(Calendar.YEAR), cal.get(Calendar.MONTH) + 1, cal.get(Calendar.DAY_OF_MONTH),
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND)  * 1000000,
+                    java.time.ZoneOffset.ofTotalSeconds((cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET)) / 1000));
+        } else if (sourceObject instanceof Long) {
+            offsetDateTime = java.time.OffsetDateTime.ofInstant(
+                    java.time.Instant.ofEpochSecond((Long) sourceObject), getDefaultZoneOffset());
+        } else {
+            throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.TIME_ODATETIME);
+        }
+
+        return offsetDateTime;
+    }
+
+    /**
+     * INTERNAL: Build a valid instance of java.time.OffsetTime from the given
+     * source object.
+     *
+     * @param sourceObject
+     *            Valid object of class java.sql.Timestamp, String,
+     *            java.util.Date, or Long
+     */
+    protected java.time.OffsetTime convertObjectToOffsetTime(Object sourceObject) throws ConversionException {
+        java.time.OffsetTime offsetTime = null;
+
+        if (sourceObject instanceof java.time.OffsetTime) {
+            return (java.time.OffsetTime) sourceObject;
+        }
+
+        if (sourceObject instanceof String) {
+            try {
+                offsetTime = java.time.OffsetTime.parse(((String) sourceObject).replace(' ', 'T'), Helper.getDefaultDateTimeFormatter());
+            } catch (Exception e) {
+                java.time.LocalTime localTime = java.time.LocalTime.parse(((String) sourceObject).replace(' ', 'T'), Helper.getDefaultDateTimeFormatter());
+                offsetTime = java.time.OffsetTime.of(localTime, java.time.OffsetDateTime.now().getOffset());
+            }
+        } else if (sourceObject instanceof java.util.Date) {
+            // handles sql.Time, sql.Date, sql.Timestamp
+            Calendar cal = Helper.allocateCalendar();
+            cal.setTime((java.util.Date) sourceObject);
+            offsetTime = java.time.OffsetTime.of(
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND),
+                    java.time.ZoneOffset.ofTotalSeconds((cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET)) / 1000));
+            Helper.releaseCalendar(cal);
+        } else if (sourceObject instanceof Calendar) {
+            Calendar cal = (Calendar) sourceObject;
+            offsetTime = java.time.OffsetTime.of(
+                    cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND), cal.get(Calendar.MILLISECOND),
+                    java.time.ZoneOffset.ofTotalSeconds((cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET)) / 1000));
+        } else if (sourceObject instanceof Long) {
+            offsetTime = java.time.OffsetTime.ofInstant(
+                    java.time.Instant.ofEpochSecond((Long) sourceObject), getDefaultZoneOffset());
+        } else {
+            throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.TIME_OTIME);
+        }
+
+        return offsetTime;
+    }
+
     /**
      * INTERNAL:
      * Build a valid instance of java.net.URL from the given source object.
@@ -796,6 +1086,8 @@ public class ConversionManager extends CoreConversionManager implements Serializ
             date = Helper.utilDateFromLong((Long)sourceObject);
         } else if (sourceObject instanceof java.util.Date) {
             date = new java.util.Date(((java.util.Date) sourceObject).getTime());
+        } else if (sourceObject instanceof LocalDateTime) {
+            date = Helper.utilDateFromTimestamp(java.sql.Timestamp.valueOf((LocalDateTime) sourceObject));
         } else {
             throw ConversionException.couldNotBeConverted(sourceObject, ClassConstants.UTILDATE);
         }
@@ -886,7 +1178,7 @@ public class ConversionManager extends CoreConversionManager implements Serializ
     public boolean hasDefaultNullValues(){
         return this.defaultNullValues != null;
     }
-    
+
     /**
      * INTERNAL:
      * Load the class using the default managers class loader.
@@ -987,7 +1279,7 @@ public class ConversionManager extends CoreConversionManager implements Serializ
      */
     public void setDefaultNullValue(Class theClass, Object theValue) {
         if (this.defaultNullValues == null){
-            this.defaultNullValues = new HashMap(5); 
+            this.defaultNullValues = new HashMap(5);
         }
         getDefaultNullValues().put(theClass, theValue);
     }
