@@ -1,21 +1,25 @@
-/*******************************************************************************
- * Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 2011, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     tware - initial implementation as part of extensibility feature
- *     01/11/2013-2.5 Guy Pelletier 
- *       - 389090: JPA 2.1 DDL Generation Support
- *     01/16/2013-2.5 Guy Pelletier 
- *       - 389090: JPA 2.1 DDL Generation Support
- *     02/04/2013-2.5 Guy Pelletier 
- *       - 389090: JPA 2.1 DDL Generation Support
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     tware - initial implementation as part of extensibility feature
+//     01/11/2013-2.5 Guy Pelletier
+//       - 389090: JPA 2.1 DDL Generation Support
+//     01/16/2013-2.5 Guy Pelletier
+//       - 389090: JPA 2.1 DDL Generation Support
+//     02/04/2013-2.5 Guy Pelletier
+//       - 389090: JPA 2.1 DDL Generation Support
+//     05/26/2016-2.7 Tomas Kraus
+//       - 494610: Session Properties map should be Map<String, Object>
 package org.eclipse.persistence.internal.jpa;
 
 import java.util.HashMap;
@@ -42,6 +46,7 @@ import org.eclipse.persistence.internal.indirection.IndirectionPolicy;
 import org.eclipse.persistence.internal.localization.ExceptionLocalization;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.sessions.DatabaseSessionImpl;
+import org.eclipse.persistence.internal.sessions.coordination.MetadataRefreshCommand;
 import org.eclipse.persistence.jpa.JpaEntityManagerFactory;
 import org.eclipse.persistence.mappings.DatabaseMapping;
 import org.eclipse.persistence.mappings.ForeignReferenceMapping;
@@ -55,7 +60,6 @@ import org.eclipse.persistence.sessions.Session;
 import org.eclipse.persistence.sessions.UnitOfWork.CommitOrderType;
 import org.eclipse.persistence.sessions.broker.SessionBroker;
 import org.eclipse.persistence.sessions.coordination.CommandManager;
-import org.eclipse.persistence.internal.sessions.coordination.MetadataRefreshCommand;
 import org.eclipse.persistence.sessions.factories.SessionManager;
 import org.eclipse.persistence.sessions.server.Server;
 import org.eclipse.persistence.sessions.server.ServerSession;
@@ -70,13 +74,13 @@ import org.eclipse.persistence.sessions.server.ServerSession;
  */
 public class EntityManagerFactoryImpl implements EntityManagerFactory, PersistenceUnitUtil, JpaEntityManagerFactory {
     protected EntityManagerFactoryDelegate delegate;
-    
-    
+
+
     /**
      * Returns the id of the entity. A generated id is not guaranteed to be
      * available until after the database insert has occurred. Returns null if
      * the entity does not yet have an id
-     * 
+     *
      * @param entity
      * @return id of the entity
      * @throws IllegalArgumentException
@@ -94,7 +98,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
             throw new IllegalArgumentException(ExceptionLocalization.buildMessage("jpa_persistence_util_non_persistent_class", new Object[] { entity }));
         }
     }
-    
+
 
     /**
      * Determine the load state of an entity belonging to the persistence unit.
@@ -104,7 +108,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * isLoaded(Object, String) method should be used to determine the load
      * state of an attribute. Not doing so might lead to unintended loading of
      * state.
-     * 
+     *
      * @param entity
      *            whose load state is to be determined
      * @return false if the entity has not been loaded, else true.
@@ -129,7 +133,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Determine the load state of a given persistent attribute of an entity
      * belonging to the persistence unit.
-     * 
+     *
      * @param entity
      *            containing the attribute
      * @param attributeName
@@ -157,11 +161,11 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Check whether a named attribute on a given entity with a given mapping
      * has been loaded.
-     * 
+     *
      * This method will check the valueholder or indirect collection for LAZY
      * ForeignReferenceMappings to see if has been instantiated and otherwise
      * check the fetch group.
-     * 
+     *
      * @param entity
      * @param attributeName
      * @param mapping
@@ -186,17 +190,17 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Will return an instance of the Factory. Should only be called by
      * EclipseLink.
-     * 
+     *
      * @param serverSession
      */
-    public EntityManagerFactoryImpl(AbstractSession session) {
-        delegate = new EntityManagerFactoryDelegate(session, this);
+    public EntityManagerFactoryImpl(AbstractSession serverSession) {
+        delegate = new EntityManagerFactoryDelegate(serverSession, this);
     }
 
     public EntityManagerFactoryImpl(EntityManagerSetupImpl setupImpl, Map properties) {
         delegate = new EntityManagerFactoryDelegate(setupImpl, properties, this);
     }
-    
+
     /**
      * Create a dynamic persistence unit which does not use the persistence.xml.
      * Instead all configuration is driven from the provided persistence unit
@@ -205,15 +209,16 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     public EntityManagerFactoryImpl(String persistenceUnitName, Map<String, Object> properties, List<ClassDescriptor> descriptors) {
         delegate = new EntityManagerFactoryDelegate(persistenceUnitName, properties, descriptors, this);
     }
-    
+
     /**
      * ADVANCED:
      * Re-bootstrap this factory.  This method will rebuild the EntityManagerFactory.  It should be used
-     * in conjunction with a MetadataSource to allow mappings to be changed in a running system.  All existing 
+     * in conjunction with a MetadataSource to allow mappings to be changed in a running system.  All existing
      * EntityMangers will continue to function with the old metadata, but new factories will use the new metadata.
      * @param properties
      */
-    public void refreshMetadata(Map properties){
+    @Override
+    public void refreshMetadata(Map<String, Object> properties){
         EntityManagerSetupImpl setupImpl = delegate.getSetupImpl();
         if (setupImpl == null){
             throw PersistenceUnitLoadingException.cannotRefreshEntityManagerFactoryCreatedFromSession(delegate.getServerSession().getName());
@@ -256,7 +261,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
             throw ex;
         }
     }
-    
+
     /**
      * INTERNAL: Returns the SessionBroker that the Factory will be using and
      * initializes it if it is not available. This method makes use of the
@@ -264,16 +269,18 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * construction
      * TODO: should throw IllegalStateException if not SessionBroker
      */
+    @Override
     public SessionBroker getSessionBroker() {
         return delegate.getSessionBroker();
     }
-    
+
     /**
      * INTERNAL: Returns the ServerSession that the Factory will be using and
      * initializes it if it is not available. This method makes use of the
      * partially constructed session stored in our setupImpl and completes its
      * construction
      */
+    @Override
     public ServerSession getServerSession() {
         return delegate.getServerSession();
     }
@@ -284,6 +291,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * throw an {@link IllegalStateException}, except for {@link #isOpen}, which
      * will return <code>false</code>.
      */
+    @Override
     public synchronized void close() {
         delegate.close();
     }
@@ -292,6 +300,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * Indicates whether or not this factory is open. Returns <code>true</code>
      * until a call to {@link #close} is made.
      */
+    @Override
     public boolean isOpen() {
         return delegate.isOpen();
     }
@@ -299,6 +308,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * PUBLIC: Returns an EntityManager for this deployment.
      */
+    @Override
     public EntityManager createEntityManager() {
         return createEntityManagerImpl(null, null);
     }
@@ -306,21 +316,24 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * PUBLIC: Returns an EntityManager for this deployment.
      */
+    @Override
     public EntityManager createEntityManager(Map properties) {
         return createEntityManagerImpl(properties, null);
     }
 
+    @Override
     public EntityManager createEntityManager(SynchronizationType synchronizationType) {
         return createEntityManagerImpl(null, synchronizationType);
     }
-    
+
+    @Override
     public EntityManager createEntityManager(SynchronizationType synchronizationType, Map map) {
         return createEntityManagerImpl(map, synchronizationType);
     }
-    
+
     protected EntityManagerImpl createEntityManagerImpl(Map properties, SynchronizationType syncType) {
         EntityManagerSetupImpl setupImpl = delegate.getSetupImpl();
-        
+
         if (setupImpl != null) {
             if (setupImpl.isMetadataExpired()) {
                 String sessionName = setupImpl.getSessionName();
@@ -333,7 +346,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
                 }
             }
         }
-        
+
         return delegate.createEntityManagerImpl(properties, syncType);
     }
 
@@ -344,18 +357,15 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * to refresh will not affect that metadata on this EntityManagerFactory.
      * @return
      */
+    @Override
     public EntityManagerFactoryDelegate unwrap(){
         return delegate;
     }
-    
+
     protected void verifyOpen() {
         delegate.verifyOpen();
     }
 
-    protected void finalize() throws Throwable {
-        delegate = null;
-    }
-
     /**
      * The method return user defined property passed in from
      * EntityManagerFactory.
@@ -435,11 +445,12 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Return interface providing access to utility methods for the persistence
      * unit.
-     * 
+     *
      * @return PersistenceUnitUtil interface
      * @throws IllegalStateException
      *             if the entity manager factory has been closed.
      */
+    @Override
     public PersistenceUnitUtil getPersistenceUnitUtil() {
         return delegate.getPersistenceUnitUtil();
     }
@@ -471,7 +482,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Return the default FlashClearCache mode to be used. Relevant only in case
      * call to flush method followed by call to clear method.
-     * 
+     *
      * @see org.eclipse.persistence.config.FlushClearCache
      */
     public String getFlushClearCache() {
@@ -481,7 +492,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Set the default FlashClearCache mode to be used. Relevant only in case
      * call to flush method followed by call to clear method.
-     * 
+     *
      * @see org.eclipse.persistence.config.FlushClearCache
      */
     public void setFlushClearCache(String flushClearCache) {
@@ -505,13 +516,14 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     }
 
     /**
-     * Access the cache that is associated with the entity manager 
+     * Access the cache that is associated with the entity manager
      * factory (the "second level cache").
      * @return instance of the <code>Cache</code> interface
      * @throws IllegalStateException if the entity manager factory has been closed
      * @see javax.persistence.EntityManagerFactory#getCache()
      * @since Java Persistence 2.0
      */
+    @Override
     public Cache getCache() {
         return delegate.getCache();
     }
@@ -520,10 +532,12 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * @see javax.persistence.EntityManagerFactory#getProperties()
      * @since Java Persistence API 2.0
      */
+    @Override
     public Map<String, Object> getProperties() {
         return delegate.getProperties();
     }
 
+    @Override
     public DatabaseSessionImpl getDatabaseSession() {
         return delegate.getDatabaseSession();
     }
@@ -531,6 +545,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * @see javax.persistence.EntityManagerFactory#getCriteriaBuilder()
      * @since Java Persistence 2.0
      */
+    @Override
     public CriteriaBuilder getCriteriaBuilder() {
         return delegate.getCriteriaBuilder();
     }
@@ -538,13 +553,14 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Return an instance of Metamodel interface for access to the metamodel of
      * the persistence unit.
-     * 
+     *
      * @return Metamodel instance
      * @throws IllegalStateException
      *             if the entity manager factory has been closed.
      * @see javax.persistence.EntityManagerFactory#getMetamodel()
      * @since Java Persistence 2.0
      */
+    @Override
     public Metamodel getMetamodel() {
         return delegate.getMetamodel();
     }
@@ -553,7 +569,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * INTERNAL: Convenience function to allow us to reset the Metamodel in the
      * possible case that we want to regenerate it. This function is outside of
      * the JPA 2.0 specification.
-     * 
+     *
      * @param aMetamodel
      * @since Java Persistence 2.0
      */
@@ -564,7 +580,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     /**
      * Determine the load state of a given persistent attribute of an entity
      * belonging to the persistence unit.
-     * 
+     *
      * @param entity
      *            containing the attribute
      * @param attributeName
@@ -572,6 +588,7 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * @return false if entity's state has not been loaded or if the attribute
      *         state has not been loaded, otherwise true
      */
+    @Override
     public boolean isLoaded(Object entity, String attributeName) {
         return delegate.isLoaded(entity, attributeName);
     }
@@ -584,11 +601,12 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * isLoaded(Object, String) method should be used to determine the load
      * state of an attribute. Not doing so might lead to unintended loading of
      * state.
-     * 
+     *
      * @param entity
      *            whose load state is to be determined
      * @return false if the entity has not been loaded, else true.
      */
+    @Override
     public boolean isLoaded(Object entity) {
         return delegate.isLoaded(entity);
     }
@@ -597,12 +615,13 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
      * Returns the id of the entity. A generated id is not guaranteed to be
      * available until after the database insert has occurred. Returns null if
      * the entity does not yet have an id
-     * 
+     *
      * @param entity
      * @return id of the entity
      * @throws IllegalStateException
      *             if the entity is found not to be an entity.
      */
+    @Override
     public Object getIdentifier(Object entity) {
         return delegate.getIdentifier(entity);
     }
@@ -613,14 +632,15 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
     public CommitOrderType getCommitOrder() {
         return delegate.getCommitOrder();
     }
-    
+
     /**
      * Set updates should be ordered by primary key to avoid possible database deadlocks.
      */
     public void setCommitOrder(CommitOrderType commitOrder) {
         delegate.setCommitOrder(commitOrder);
     }
-    
+
+    @Override
     public void addNamedQuery(String name, Query query) {
         QueryImpl queryImpl = query.unwrap(QueryImpl.class);
         DatabaseQuery unwrapped = (DatabaseQuery) queryImpl.getDatabaseQueryInternal().clone();
@@ -634,23 +654,25 @@ public class EntityManagerFactoryImpl implements EntityManagerFactory, Persisten
         this.getServerSession().addQuery(name, unwrapped, true);
     }
 
+    @Override
     public <T> T unwrap(Class<T> cls) {
         if (cls.equals(JpaEntityManagerFactory.class) || cls.equals(EntityManagerFactoryImpl.class)) {
             return (T) this;
         }else if (cls.equals(EntityManagerFactoryDelegate.class)) {
             return (T) this.delegate;
-        }else if (cls.equals(Session.class) || cls.equals(AbstractSession.class)) {            
+        }else if (cls.equals(Session.class) || cls.equals(AbstractSession.class)) {
             return (T) this.delegate.getAbstractSession();
-        } else if (cls.equals(DatabaseSession.class) || cls.equals(DatabaseSessionImpl.class)) {            
+        } else if (cls.equals(DatabaseSession.class) || cls.equals(DatabaseSessionImpl.class)) {
             return (T) this.getDatabaseSession();
-        } else if (cls.equals(Server.class) || cls.equals(ServerSession.class)) {            
+        } else if (cls.equals(Server.class) || cls.equals(ServerSession.class)) {
             return (T) this.getServerSession();
-        } else if (cls.equals(SessionBroker.class)) {            
+        } else if (cls.equals(SessionBroker.class)) {
             return (T) this.getSessionBroker();
         }
         throw new PersistenceException(ExceptionLocalization.buildMessage("unable_to_unwrap_jpa", new String[]{EntityManagerFactory.class.getName(),cls.getName()}));
     }
 
+    @Override
     public <T> void addNamedEntityGraph(String graphName, EntityGraph<T> entityGraph) {
         AttributeGroup group = ((EntityGraphImpl)entityGraph).getAttributeGroup().clone();
         group.setName(graphName);
