@@ -1,22 +1,24 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2017 Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     05/16/2008-1.0M8 Guy Pelletier 
- *       - 218084: Implement metadata merging functionality between mapping files
- *     01/05/2010-2.1 Guy Pelletier 
- *       - 211324: Add additional event(s) support to the EclipseLink-ORM.XML Schema
- *     07/15/2010-2.2 Guy Pelletier
- *       -311395 : Multiple lifecycle callback methods for the same lifecycle event
- *     12/14/2017-2.6.6 Tomas Kraus
- *       - 291546: Performance degradation due to usage of Vector in DescriptorEventManager
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     05/16/2008-1.0M8 Guy Pelletier
+//       - 218084: Implement metadata merging functionality between mapping files
+//     01/05/2010-2.1 Guy Pelletier
+//       - 211324: Add additional event(s) support to the EclipseLink-ORM.XML Schema
+//     07/15/2010-2.2 Guy Pelletier
+//       -311395 : Multiple lifecycle callback methods for the same lifecycle event
+//     12/14/2017-3.0 Tomas Kraus
+//       - 291546: Performance degradation due to usage of Vector in DescriptorEventManager
 package org.eclipse.persistence.internal.jpa.metadata.listeners;
 
 import java.lang.reflect.InvocationTargetException;
@@ -42,14 +44,14 @@ import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.logging.SessionLog;
 
 /**
- * An EntityListener is placed on the owning entity's descriptor. 
- * Callback methods from an EntityListener require a signature on the method. 
+ * An EntityListener is placed on the owning entity's descriptor.
+ * Callback methods from an EntityListener require a signature on the method.
  * Namely, they must have an Object parameter.
- * 
+ *
  * @author Guy Pelletier
  * @since Eclipselink 1.0
  */
-public class EntityListener extends DescriptorEventAdapter {
+public class EntityListener<T> extends DescriptorEventAdapter {
     public final static String POST_BUILD = "postBuild";
     public final static String POST_CLONE = "postClone";
     public final static String POST_DELETE = "postDelete";
@@ -59,9 +61,9 @@ public class EntityListener extends DescriptorEventAdapter {
     public final static String PRE_PERSIST = "prePersist";
     public final static String PRE_REMOVE = "preRemove";
     public final static String PRE_UPDATE_WITH_CHANGES = "preUpdateWithChanges";
-    
-    private Object m_listener;
-    private Class m_listenerClass;
+
+    private T m_listener;
+    private Class<T> m_listenerClass;
     private Class m_entityClass;
     private Map<String, List<Method>> m_methods;
     private final Map<String, Map<Integer, Boolean>> m_overriddenEvents;
@@ -82,7 +84,6 @@ public class EntityListener extends DescriptorEventAdapter {
         mappings.put(DescriptorEventManager.PreUpdateWithChangesEvent, PRE_UPDATE_WITH_CHANGES);
         m_eventStrings = Collections.unmodifiableMap(mappings);
     }
-
     /**
      * INTERNAL:
      */
@@ -90,16 +91,16 @@ public class EntityListener extends DescriptorEventAdapter {
         m_entityClass = entityClass;
         m_methods = new ConcurrentHashMap<>();
 
-        // Remember which events are overridden in subclasses. Overriden events
+        // Remember which events are overridden in subclasses. Overridden events
         // must be built for each subclass chain.
         m_overriddenEvents = new ConcurrentHashMap<>();
     }
 
-    public EntityListener(Class listenerClass, Class entityClass){
+    public EntityListener(Class<T> listenerClass, Class entityClass){
         this(entityClass);
         this.m_listenerClass = listenerClass;
     }
-    
+
     /**
      * INTERNAL:
      * You can have multiple event methods for the same event, however, only
@@ -108,7 +109,7 @@ public class EntityListener extends DescriptorEventAdapter {
     public void addEventMethod(String event, Method method) {
         if (m_methods.containsKey(event)) {
             Method lastEventMethod = getLastEventMethod(event);
-            
+
             if (lastEventMethod.getDeclaringClass().equals(method.getDeclaringClass())) {
                 throw ValidationException.multipleLifecycleCallbackMethodsForSameLifecycleEvent(getListenerClass(), method, lastEventMethod);
             } else {
@@ -120,41 +121,41 @@ public class EntityListener extends DescriptorEventAdapter {
         } else {
             // Validate the method is valid.
             validateMethod(method);
-            
+
             // Create the methods list and add this method to it.
-            List methods = new ArrayList<Method>();
+            List<Method> methods = new ArrayList<>();
             methods.add(method);
             m_methods.put(event, methods);
         }
     }
-    
+
     /**
      * Create the wrapped listener and trigger CDI injection.
-     * @param entityListenerClass
-     * @return the class instance that has had injection run on it. If injection failes, null.
+     * @param entityListenerClass the {@link EntityListener} class
+     * @return the class instance that has had injection run on it. If injection fails, null.
      */
-    protected Object createEntityListenerAndInjectDependancies(Class entityListenerClass){
+    protected T createEntityListenerAndInjectDependencies(Class<T> entityListenerClass){
         try{
-            return owningSession.getEntityListenerInjectionManager().createEntityListenerAndInjectDependancies(entityListenerClass);
+            return owningSession.<T>getInjectionManager().createManagedBeanAndInjectDependencies(entityListenerClass);
         } catch (Exception e){
             owningSession.logThrowable(SessionLog.FINEST, SessionLog.JPA, e);
         }
         return null;
     }
-    
+
     /**
      * Construct an instance of the wrapped entity listener
      * This method will attempt to create the listener in a CDI injection
      * friendly manner and if that fails, reflectively instantiate the class
-     * @return
+     * @return the entity listener instance
      */
-    protected Object constructListenerInstance(AbstractSession session){
-        Object entityListenerClassInstance =  createEntityListenerAndInjectDependancies(m_listenerClass);
+    protected T constructListenerInstance(){
+        T entityListenerClassInstance =  createEntityListenerAndInjectDependencies(m_listenerClass);
         try {
             if (PrivilegedAccessHelper.shouldUsePrivilegedAccess()){
                 try {
                     if (entityListenerClassInstance == null){
-                        entityListenerClassInstance = AccessController.doPrivileged(new PrivilegedNewInstanceFromClass(m_listenerClass));
+                        entityListenerClassInstance = AccessController.doPrivileged(new PrivilegedNewInstanceFromClass<>(m_listenerClass));
                     }
                 } catch (PrivilegedActionException exception) {
                     throw ValidationException.errorInstantiatingClass(m_listenerClass, exception.getException());
@@ -164,21 +165,19 @@ public class EntityListener extends DescriptorEventAdapter {
                     entityListenerClassInstance = PrivilegedAccessHelper.newInstanceFromClass(m_listenerClass);
                 }
             }
-        } catch (IllegalAccessException exception) {
-            throw ValidationException.errorInstantiatingClass(m_listenerClass, exception);
-        } catch (InstantiationException exception) {
+        } catch (IllegalAccessException | InstantiationException exception) {
             throw ValidationException.errorInstantiatingClass(m_listenerClass, exception);
         }
         return entityListenerClassInstance;
     }
-    
+
     /**
      * INTERNAL:
      */
     public Class getEntityClass() {
         return m_entityClass;
     }
-    
+
     /**
      * INTERNAL:
      */
@@ -199,7 +198,7 @@ public class EntityListener extends DescriptorEventAdapter {
     public void setOwningSession(AbstractSession owningSession) {
         this.owningSession = owningSession;
     }
-    
+
     /**
      * INTERNAL:
      */
@@ -214,7 +213,7 @@ public class EntityListener extends DescriptorEventAdapter {
     protected List<Method> getEventMethods(String event) {
         return m_methods.get(event);
     }
-    
+
     /**
      * INTERNAL:
      * Assumes a check for event methods for the given event has been called
@@ -224,20 +223,21 @@ public class EntityListener extends DescriptorEventAdapter {
         List<Method> methods = m_methods.get(event);
         return methods.get(methods.size()-1);
     }
-    
-    public Object getListener(AbstractSession session) {
+
+    public T getListener() {
         if (m_listener == null){
-            m_listener = constructListenerInstance(session);
+            m_listener = constructListenerInstance();
         }
         return m_listener;
     }
+    
     /**
      * INTERNAL:
      */
     public Class getListenerClass() {
         return m_listenerClass;
     }
-    
+
     /**
      * INTERNAL:
      */
@@ -251,21 +251,21 @@ public class EntityListener extends DescriptorEventAdapter {
     public boolean hasCallbackMethods() {
         return m_methods.size() > 0;
     }
-    
+
     /**
      * INTERNAL:
      */
     protected boolean hasEventMethods(int eventCode) {
         return getEventMethods(eventCode) != null;
     }
-    
+
     /**
      * INTERNAL:
      */
     protected boolean hasEventMethods(String event) {
         return getEventMethods(event) != null;
     }
-    
+
     /**
      * INTERNAL:
      */
@@ -277,28 +277,28 @@ public class EntityListener extends DescriptorEventAdapter {
                 }
             }
         }
-        
+
         return false;
     }
-    
+
     /**
      * INTERNAL:
      */
     protected boolean hasOverriddenEventMethod(Method eventMethod, int eventCode) {
         return hasOverriddenEventMethod(getEventMethods(eventCode), eventMethod);
     }
-    
+
     /**
      * INTERNAL:
      */
     protected boolean hasOverriddenEventMethod(Method eventMethod, String eventCode) {
         return hasOverriddenEventMethod(getEventMethods(eventCode), eventMethod);
     }
-    
+
     /**
      * INTERNAL:
      */
-    void invokeMethod(Method method, Object onObject, Object[] objectList, DescriptorEvent event) {
+    void invokeMethod(Method method, Object onObject, Object[] objectList) {
         if (method != null) {
             try {
                 if (PrivilegedAccessHelper.shouldUsePrivilegedAccess()){
@@ -310,12 +310,12 @@ public class EntityListener extends DescriptorEventAdapter {
                             throw ValidationException.invalidCallbackMethod(onObject.getClass(), method.toString());
                         } else {
                             Throwable cause = throwableException.getCause();
-                            
+
                             if (cause instanceof RuntimeException) {
-                                throw (RuntimeException) cause;    
+                                throw (RuntimeException) cause;
                             } else {
                                 throw (Error) cause;
-                            }                         
+                            }
                         }
                     }
                 } else {
@@ -325,16 +325,16 @@ public class EntityListener extends DescriptorEventAdapter {
                 throw ValidationException.invalidCallbackMethod(onObject.getClass(), method.toString());
             } catch (InvocationTargetException e) {
                 Throwable cause = e.getCause();
-                
+
                 if (cause instanceof RuntimeException) {
-                    throw (RuntimeException) cause;    
+                    throw (RuntimeException) cause;
                 } else {
                     throw (Error) cause;
                 }
             }
         }
     }
-    
+
     /**
      * INTERNAL:
      */
@@ -343,7 +343,7 @@ public class EntityListener extends DescriptorEventAdapter {
         if (eventMethods != null) {
             for (Method method : eventMethods) {
                 Object[] objectList = { descriptorEvent.getSource() };
-                invokeMethod(method, getListener(descriptorEvent.getSession()), objectList, descriptorEvent);
+                invokeMethod(method, getListener(), objectList);
             }
         }
     }
@@ -353,6 +353,7 @@ public class EntityListener extends DescriptorEventAdapter {
      * Return true if listener has a lifecycle callback method that is
      * overridden in a subclass.
      */
+    @Override
     public boolean isOverriddenEvent(DescriptorEvent event, List<DescriptorEventManager> eventManagers) {
         int eventCode = event.getEventCode();
         String forSubclass = event.getDescriptor().getJavaClassName();
@@ -372,7 +373,7 @@ public class EntityListener extends DescriptorEventAdapter {
                     for (DescriptorEventManager eventManager : eventManagers) {
                         EntityListener childListener = (EntityListener) eventManager.getEntityEventListener();
 
-                        // We can't override ourself ...
+                        // We can't override ourselves ...
                         if (childListener == this) {
                             break;
                         } else {
@@ -388,45 +389,50 @@ public class EntityListener extends DescriptorEventAdapter {
                 }
             }
             subClassMap.put(eventCode, hasOverrides);
-            //putting this here prevents a vm reorder from putting an unbuilt Map in the 
+            //putting this here prevents a vm reorder from putting an unbuilt Map in the
             //m_overriddenEvents collection
             m_overriddenEvents.put(forSubclass, subClassMap);
         }
-        
+
         return subClassMap.get(eventCode);
     }
-    
+
     /**
      * INTERNAL:
      */
+    @Override
     public void postBuild(DescriptorEvent event) {
         invokeMethod(POST_BUILD, event);
     }
-    
+
     /**
      * INTERNAL:
      */
+    @Override
     public void postClone(DescriptorEvent event) {
         invokeMethod(POST_CLONE, event);
     }
-    
+
     /**
      * INTERNAL:
      */
+    @Override
     public void postDelete(DescriptorEvent event) {
         invokeMethod(POST_DELETE, event);
     }
-    
+
     /**
      * INTERNAL:
      */
+    @Override
     public void postInsert(DescriptorEvent event) {
         invokeMethod(POST_INSERT, event);
     }
-    
+
     /**
      * INTERNAL:
      */
+    @Override
     public void postRefresh(DescriptorEvent event) {
         invokeMethod(POST_REFRESH, event);
     }
@@ -434,6 +440,7 @@ public class EntityListener extends DescriptorEventAdapter {
     /**
      * INTERNAL:
      */
+    @Override
     public void postUpdate(DescriptorEvent event) {
         invokeMethod(POST_UPDATE, event);
     }
@@ -441,20 +448,23 @@ public class EntityListener extends DescriptorEventAdapter {
     /**
      * INTERNAL:
      */
+    @Override
     public void prePersist(DescriptorEvent event) {
         invokeMethod(PRE_PERSIST, event);
     }
-    
+
     /**
      * INTERNAL:
      */
+    @Override
     public void preRemove(DescriptorEvent event) {
         invokeMethod(PRE_REMOVE, event);
     }
-    
+
     /**
      * INTERNAL:
      */
+    @Override
     public void preUpdateWithChanges(DescriptorEvent event) {
         invokeMethod(PRE_UPDATE_WITH_CHANGES, event);
     }
@@ -465,7 +475,7 @@ public class EntityListener extends DescriptorEventAdapter {
     public void setPostBuildMethod(Method method) {
         addEventMethod(POST_BUILD, method);
     }
-    
+
     /**
      * INTERNAL:
      */
@@ -486,14 +496,14 @@ public class EntityListener extends DescriptorEventAdapter {
     public void setPostInsertMethod(Method method) {
         addEventMethod(POST_INSERT, method);
     }
-    
+
     /**
      * INTERNAL:
      */
     public void setPostRefreshMethod(Method method) {
         addEventMethod(POST_REFRESH, method);
     }
-    
+
     /**
      * INTERNAL:
      */
@@ -507,35 +517,36 @@ public class EntityListener extends DescriptorEventAdapter {
     public void setPrePersistMethod(Method method) {
         addEventMethod(PRE_PERSIST, method);
     }
-    
+
     /**
      * INTERNAL:
      */
     public void setPreRemoveMethod(Method method) {
         addEventMethod(PRE_REMOVE, method);
     }
-    
+
     /**
      * INTERNAL:
      */
-    public void setPreUpdateWithChangesMethod(Method method) { 
+    public void setPreUpdateWithChangesMethod(Method method) {
         addEventMethod(PRE_UPDATE_WITH_CHANGES, method);
     }
- 
+
     /**
      * INTERNAL:
      * Used in the debugger.
      */
+    @Override
     public String toString() {
         return getEntityClass().getName();
     }
-    
+
     /**
      * INTERNAL:
      */
     protected void validateMethod(Method method) {
         int numberOfParameters = method.getParameterTypes().length;
-           
+
         if (numberOfParameters == 1 && method.getParameterTypes()[0].isAssignableFrom(m_entityClass)) {
             // So far so good, now check the method modifiers.
             validateMethodModifiers(method);
@@ -544,13 +555,13 @@ public class EntityListener extends DescriptorEventAdapter {
             throw ValidationException.invalidEntityListenerCallbackMethodArguments(m_entityClass, parameterClass, getListenerClass(), method.getName());
         }
     }
-    
+
     /**
      * INTERNAL:
      */
     protected void validateMethodModifiers(Method method) {
         int modifiers = method.getModifiers();
-        
+
         if (Modifier.isStatic(modifiers) || Modifier.isFinal(modifiers)) {
             throw ValidationException.invalidCallbackMethodModifier(getListenerClass(), method.getName());
         }
