@@ -88,7 +88,8 @@ public class JSRInlinerAdapter extends MethodNode implements Opcodes {
    * @param name the method's name.
    * @param descriptor the method's descriptor.
    * @param signature the method's signature. May be {@literal null}.
-   * @param exceptions the internal names of the method's exception classes. May be {@literal null}.
+   * @param exceptions the internal names of the method's exception classes (see {@link
+   *     org.eclipse.persistence.internal.libraries.asm.Type#getInternalName()}). May be {@literal null}.
    * @throws IllegalStateException if a subclass calls this constructor.
    */
   public JSRInlinerAdapter(
@@ -123,7 +124,8 @@ public class JSRInlinerAdapter extends MethodNode implements Opcodes {
    * @param name the method's name.
    * @param descriptor the method's descriptor.
    * @param signature the method's signature. May be {@literal null}.
-   * @param exceptions the internal names of the method's exception classes. May be {@literal null}.
+   * @param exceptions the internal names of the method's exception classes (see {@link
+   *     org.eclipse.persistence.internal.libraries.asm.Type#getInternalName()}). May be {@literal null}.
    */
   protected JSRInlinerAdapter(
       final int api,
@@ -423,7 +425,7 @@ public class JSRInlinerAdapter extends MethodNode implements Opcodes {
   }
 
   /** An instantiation of a subroutine. */
-  private class Instantiation extends AbstractMap<LabelNode, LabelNode> {
+  private final class Instantiation extends AbstractMap<LabelNode, LabelNode> {
 
     /**
      * The instantiation from which this one was created (or {@literal null} for the instantiation
