@@ -49,7 +49,7 @@ public abstract class Remapper {
    * @param descriptor a type descriptor.
    * @return the given descriptor, with its [array element type] internal name remapped with {@link
    *     #map(String)} (if the descriptor corresponds to an array or object type, otherwise the
-   *     descriptor is returned as is).
+   *     descriptor is returned as is). See {@link Type#getInternalName()}.
    */
   public String mapDesc(final String descriptor) {
     return mapType(Type.getType(descriptor)).getDescriptor();
@@ -63,7 +63,7 @@ public abstract class Remapper {
    * @return the given type, with its [array element type] internal name remapped with {@link
    *     #map(String)} (if the type is an array or object type, otherwise the type is returned as
    *     is) or, of the type is a method type, with its descriptor remapped with {@link
-   *     #mapMethodDesc(String)}.
+   *     #mapMethodDesc(String)}. See {@link Type#getInternalName()}.
    */
   private Type mapType(final Type type) {
     switch (type.getSort()) {
@@ -87,8 +87,10 @@ public abstract class Remapper {
   /**
    * Returns the given internal name, remapped with {@link #map(String)}.
    *
-   * @param internalName the internal name (or array type descriptor) of some (array) class.
-   * @return the given internal name, remapped with {@link #map(String)}.
+   * @param internalName the internal name (or array type descriptor) of some (array) class (see
+   *     {@link Type#getInternalName()}).
+   * @return the given internal name, remapped with {@link #map(String)} (see {@link
+   *     Type#getInternalName()}).
    */
   public String mapType(final String internalName) {
     if (internalName == null) {
@@ -100,8 +102,10 @@ public abstract class Remapper {
   /**
    * Returns the given internal names, remapped with {@link #map(String)}.
    *
-   * @param internalNames the internal names (or array type descriptors) of some (array) classes.
-   * @return the given internal name, remapped with {@link #map(String)}.
+   * @param internalNames the internal names (or array type descriptors) of some (array) classes
+   *     (see {@link Type#getInternalName()}).
+   * @return the given internal name, remapped with {@link #map(String)} (see {@link
+   *     Type#getInternalName()}).
    */
   public String[] mapTypes(final String[] internalNames) {
     String[] remappedInternalNames = null;
@@ -255,14 +259,30 @@ public abstract class Remapper {
    * strategy that will work for inner classes produced by Java, but not necessarily other
    * languages. Subclasses can override.
    *
-   * @param name the fully-qualified internal name of the inner class.
-   * @param ownerName the internal name of the owner class of the inner class.
-   * @param innerName the internal name of the inner class.
+   * @param name the fully-qualified internal name of the inner class (see {@link
+   *     Type#getInternalName()}).
+   * @param ownerName the internal name of the owner class of the inner class (see {@link
+   *     Type#getInternalName()}).
+   * @param innerName the internal name of the inner class (see {@link Type#getInternalName()}).
    * @return the new inner name of the inner class.
    */
   public String mapInnerClassName(
       final String name, final String ownerName, final String innerName) {
     final String remappedInnerName = this.mapType(name);
+
+    if (remappedInnerName.equals(name)) {
+      return innerName;
+    } else {
+      int originSplit = name.lastIndexOf('/');
+      int remappedSplit = remappedInnerName.lastIndexOf('/');
+      if (originSplit != -1 && remappedSplit != -1) {
+        if (name.substring(originSplit).equals(remappedInnerName.substring(remappedSplit))) {
+          // class name not changed
+          return innerName;
+        }
+      }
+    }
+
     if (remappedInnerName.contains("$")) {
       int index = remappedInnerName.lastIndexOf('$') + 1;
       while (index < remappedInnerName.length()
@@ -279,7 +299,8 @@ public abstract class Remapper {
    * Maps a method name to its new name. The default implementation of this method returns the given
    * name, unchanged. Subclasses can override.
    *
-   * @param owner the internal name of the owner class of the method.
+   * @param owner the internal name of the owner class of the method (see {@link
+   *     Type#getInternalName()}).
    * @param name the name of the method.
    * @param descriptor the descriptor of the method.
    * @return the new name of the method.
@@ -304,7 +325,8 @@ public abstract class Remapper {
    * Maps a record component name to its new name. The default implementation of this method returns
    * the given name, unchanged. Subclasses can override.
    *
-   * @param owner the internal name of the owner class of the field.
+   * @param owner the internal name of the owner class of the field (see {@link
+   *     Type#getInternalName()}).
    * @param name the name of the field.
    * @param descriptor the descriptor of the field.
    * @return the new name of the field.
@@ -318,7 +340,8 @@ public abstract class Remapper {
    * Maps a field name to its new name. The default implementation of this method returns the given
    * name, unchanged. Subclasses can override.
    *
-   * @param owner the internal name of the owner class of the field.
+   * @param owner the internal name of the owner class of the field (see {@link
+   *     Type#getInternalName()}).
    * @param name the name of the field.
    * @param descriptor the descriptor of the field.
    * @return the new name of the field.
@@ -353,8 +376,8 @@ public abstract class Remapper {
    * Maps the internal name of a class to its new name. The default implementation of this method
    * returns the given name, unchanged. Subclasses can override.
    *
-   * @param internalName the internal name of a class.
-   * @return the new internal name.
+   * @param internalName the internal name of a class (see {@link Type#getInternalName()}).
+   * @return the new internal name (see {@link Type#getInternalName()}).
    */
   public String map(final String internalName) {
     return internalName;
