@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1998, 2020 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 1998, 2020 IBM Corporation. All rights reserved.
+ * Copyright (c) 1998, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022 IBM Corporation. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v. 2.0 which is available at
@@ -37,10 +37,8 @@ import org.eclipse.persistence.internal.databaseaccess.DatasourceCall;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.helper.DatabaseTable;
 import org.eclipse.persistence.internal.helper.Helper;
-import org.eclipse.persistence.internal.helper.NonSynchronizedVector;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
-import org.eclipse.persistence.mappings.DatabaseMapping;
 import org.eclipse.persistence.mappings.DatabaseMapping.WriteType;
 import org.eclipse.persistence.queries.ConstructorReportItem;
 import org.eclipse.persistence.queries.DatabaseQuery;
@@ -277,13 +275,26 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         return sessionToUse.executeCall(clonedCall, this.query.getTranslationRow(), this.query);
     }
 
+    /**
+     * Execute a non selecting call.
+     * @exception  DatabaseException - an error has occurred on the database.
+     * @return Returns either a {@link DatabaseCall} or Integer value, 
+     * depending on if this INSERT call needs to return generated keys
+     */
+    public Object executeNoSelect() throws DatabaseException {
+        if(((DatabaseCall)this.call).shouldReturnGeneratedKeys()) {
+            return generateKeysExecuteNoSelect();
+        }
+        return executeNoSelectCall();
+    }
+
     /**
      * Execute a non selecting call.
      * @exception  DatabaseException - an error has occurred on the database.
      * @return the row count.
      */
-    public Integer executeNoSelect() throws DatabaseException {
-        return executeNoSelectCall();
+    public DatabaseCall generateKeysExecuteNoSelect() throws DatabaseException {
+        return (DatabaseCall)executeCall();
     }
 
     /**
@@ -405,7 +416,11 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
                         updateObjectAndRowWithReturnRow(returnFields, index == 0);
                     }
                     if ((index == 0) && usesSequencing && shouldAcquireValueAfterInsert) {
-                        updateObjectAndRowWithSequenceNumber();
+                        if(result instanceof DatabaseCall) {
+                            updateObjectAndRowWithSequenceNumber((DatabaseCall) result);
+                        } else {
+                            updateObjectAndRowWithSequenceNumber();
+                        }
                     }
                 }
             }
@@ -419,7 +434,11 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
                 updateObjectAndRowWithReturnRow(returnFields, true);
             }
             if (usesSequencing && shouldAcquireValueAfterInsert) {
-                updateObjectAndRowWithSequenceNumber();
+                if(result instanceof DatabaseCall) {
+                    updateObjectAndRowWithSequenceNumber((DatabaseCall) result);
+                } else {
+                    updateObjectAndRowWithSequenceNumber();
+                }
             }
         }
 
