@@ -1,19 +1,55 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2016 Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 1998, 2021 Oracle and/or its affiliates. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- *     Marcel Valovy - 2.6 - added case insensitive unmarshalling property
- *     Dmitry Kornilov - 2.6.1 - BeanValidationHelper refactoring
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     Marcel Valovy - 2.6 - added ci unmarshalling & BV in JAXB
+//     Dmitry Kornilov - 2.6.1 - BeanValidationHelper refactoring
 package org.eclipse.persistence.jaxb;
 
+import java.awt.Image;
+import java.io.BufferedReader;
+import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.lang.reflect.GenericArrayType;
+import java.lang.reflect.Method;
+import java.lang.reflect.ParameterizedType;
+import java.lang.reflect.Type;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Comparator;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Set;
+import java.util.StringTokenizer;
+import java.util.Vector;
+import java.util.concurrent.atomic.AtomicBoolean;
+
+import javax.xml.bind.JAXBElement;
+import javax.xml.bind.Marshaller;
+import javax.xml.bind.PropertyException;
+import javax.xml.bind.SchemaOutputResolver;
+import javax.xml.bind.ValidationEvent;
+import javax.xml.bind.ValidationEventHandler;
+import javax.xml.bind.annotation.adapters.XmlAdapter;
+import javax.xml.namespace.QName;
+import javax.xml.stream.FactoryConfigurationError;
+import javax.xml.stream.XMLInputFactory;
+import javax.xml.transform.Source;
+
 import org.eclipse.persistence.core.queries.CoreAttributeGroup;
 import org.eclipse.persistence.core.sessions.CoreProject;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
@@ -38,6 +74,7 @@ import org.eclipse.persistence.internal.oxm.mappings.Descriptor;
 import org.eclipse.persistence.internal.oxm.mappings.Field;
 import org.eclipse.persistence.internal.oxm.schema.SchemaModelGenerator;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
+import org.eclipse.persistence.internal.security.PrivilegedAccessHelper.PrivilegedCallable;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.jaxb.compiler.Generator;
 import org.eclipse.persistence.jaxb.compiler.MarshalCallback;
@@ -50,6 +87,9 @@ import org.eclipse.persistence.jaxb.json.JsonSchemaOutputResolver;
 import org.eclipse.persistence.jaxb.xmlmodel.JavaType;
 import org.eclipse.persistence.jaxb.xmlmodel.XmlBindings;
 import org.eclipse.persistence.jaxb.xmlmodel.XmlBindings.JavaTypes;
+import org.eclipse.persistence.logging.AbstractSessionLog;
+import org.eclipse.persistence.logging.LogLevel;
+import org.eclipse.persistence.logging.SessionLog;
 import org.eclipse.persistence.mappings.DatabaseMapping;
 import org.eclipse.persistence.oxm.MediaType;
 import org.eclipse.persistence.oxm.NamespaceResolver;
@@ -63,41 +103,6 @@ import org.eclipse.persistence.oxm.platform.XMLPlatform;
 import org.eclipse.persistence.sessions.Project;
 import org.eclipse.persistence.sessions.SessionEventListener;
 
-import javax.xml.bind.JAXBElement;
-import javax.xml.bind.Marshaller;
-import javax.xml.bind.PropertyException;
-import javax.xml.bind.SchemaOutputResolver;
-import javax.xml.bind.ValidationEvent;
-import javax.xml.bind.ValidationEventHandler;
-import javax.xml.bind.annotation.adapters.XmlAdapter;
-import javax.xml.namespace.QName;
-import javax.xml.stream.FactoryConfigurationError;
-import javax.xml.stream.XMLInputFactory;
-import javax.xml.transform.Source;
-import java.awt.*;
-import java.io.BufferedReader;
-import java.io.InputStream;
-import java.io.InputStreamReader;
-import java.lang.reflect.GenericArrayType;
-import java.lang.reflect.Method;
-import java.lang.reflect.ParameterizedType;
-import java.lang.reflect.Type;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collection;
-import java.util.Comparator;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Map.Entry;
-import java.util.Set;
-import java.util.StringTokenizer;
-import java.util.Vector;
-import java.util.concurrent.atomic.AtomicBoolean;
-
 import static org.eclipse.persistence.jaxb.javamodel.Helper.getQualifiedJavaTypeName;
 
 /**
@@ -157,6 +162,9 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
     static {
         PARSER_FEATURES.put("http://apache.org/xml/features/validation/schema/normalized-value", false);
         PARSER_FEATURES.put("http://apache.org/xml/features/validation/schema/element-default", false);
+        if (MOXySystemProperties.moxyLoggingLevel != null) {
+            AbstractSessionLog.getLog().setLevel(LogLevel.toValue(MOXySystemProperties.moxyLoggingLevel).getId(), SessionLog.MOXY);
+        }
     }
 
     private static final String RI_XML_ACCESSOR_FACTORY_SUPPORT = "com.sun.xml.bind.XmlAccessorFactory";
@@ -167,6 +175,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
       * encountering a fatal error.
       */
     protected static final ValidationEventHandler DEFAULT_VALIDATION_EVENT_HANDLER = new ValidationEventHandler() {
+        @Override
         public boolean handleEvent(ValidationEvent event) {
             return event.getSeverity() < ValidationEvent.FATAL_ERROR;
         }
@@ -348,6 +357,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
      *
      * @param outputResolver Class that decides where the schema file (of the given namespace URI) will be written
      */
+    @Override
     public void generateSchema(SchemaOutputResolver outputResolver)  {
         if(outputResolver instanceof JsonSchemaOutputResolver) {
             generateJsonSchema(outputResolver, ((JsonSchemaOutputResolver)outputResolver).getRootClass());
@@ -412,6 +422,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
      * Create a JAXBMarshaller.  The JAXBMarshaller is used to convert Java objects
      * to XML.
      */
+    @Override
     public JAXBMarshaller createMarshaller() throws javax.xml.bind.JAXBException {
         return contextState.createMarshaller(this);
     }
@@ -420,6 +431,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
      * Create a JAXBUnmarshaller.  The JAXBUnmarshaller is used to convert XML into
      * Java objects.
      */
+    @Override
     public JAXBUnmarshaller createUnmarshaller() throws javax.xml.bind.JAXBException {
         return contextState.createUnmarshaller(this);
     }
@@ -428,6 +440,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
      * Create a JAXBValidator.  The JAXBValidator is used to validate Java objects against
      * an XSD.
      */
+    @Override
     public JAXBValidator createValidator() {
         return new JAXBValidator(getXMLContext().createValidator());
     }
@@ -435,6 +448,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
     /**
      * Create a JAXBBinder.  The JAXBBinder is used to preserve unmapped XML Data.
      */
+    @Override
     public JAXBBinder createBinder() {
         return contextState.createBinder(this);
     }
@@ -444,6 +458,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
      *
      * @param nodeClass The DOM Node class to use
      */
+    @Override
     public <T> JAXBBinder createBinder(Class<T> nodeClass) {
         if (nodeClass.getName().equals("org.w3c.dom.Node")) {
             return contextState.createBinder(this);
@@ -456,6 +471,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
      * Creates a JAXBIntrospector object.  The JAXBIntrospector allows the user to
      * access certain pieces of metadata about an instance of a JAXB bound class.
      */
+    @Override
     public JAXBIntrospector createJAXBIntrospector() {
         return new JAXBIntrospector(getXMLContext());
     }
@@ -785,6 +801,16 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
          * @param classLoader the classLoader to use.  If null then Thread.currentThread().getContextClassLoader() will be used.
          */
         public JAXBContextInput(Map properties, ClassLoader classLoader) {
+            SessionLog logger = AbstractSessionLog.getLog();
+            if (properties != null && logger.shouldLog(SessionLog.FINE, SessionLog.MOXY)) {
+                for (Map.Entry<Object, Object> item : (Set<Map.Entry<Object, Object>>)(properties.entrySet())) {
+                    if (item.getValue() == null) {
+                        logger.log(SessionLog.FINE, SessionLog.MOXY, "moxy_set_jaxb_context_property", new Object[]{item.getKey(), "NULL"});
+                    } else {
+                        logger.log(SessionLog.FINE, SessionLog.MOXY, "moxy_set_jaxb_context_property", new Object[]{item.getKey(), item.getValue()});
+                    }
+                }
+            }
             this.properties = properties;
             if (null == classLoader) {
                 this.classLoader = Thread.currentThread().getContextClassLoader();
@@ -834,6 +860,24 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
             this.contextPath = contextPath;
         }
 
+        // JaxbClassLoader creator to be used ONLY in createContextState() method
+        // statement PrivilegedAccessHelper.callDoPrivileged
+        private static final class CreateJaxbClassLoader implements PrivilegedCallable<JaxbClassLoader> {
+
+            private final ClassLoader classLoader;
+            private final Class<?>[] types;
+
+            private CreateJaxbClassLoader(final ClassLoader classLoader, final Class<?>[] types) {
+                this.classLoader = classLoader;
+                this.types = types;
+            }
+
+            @Override
+            public JaxbClassLoader call() {
+                return new JaxbClassLoader(classLoader, types);
+            }
+        }
+
         @Override
         protected JAXBContextState createContextState() throws javax.xml.bind.JAXBException {
             boolean foundMetadata = false;
@@ -892,7 +936,9 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
                 for (int i = 0; i < classes.size(); i++) {
                     classArray[i] = classes.get(i);
                 }
-                return createContextState(classArray, xmlBindingMap);
+                final CreateJaxbClassLoader createJaxbClassLoader = new CreateJaxbClassLoader(classLoader, classArray);
+                final JaxbClassLoader loader = PrivilegedAccessHelper.callDoPrivileged(createJaxbClassLoader);
+                return createContextState(loader, classArray, xmlBindingMap);
             }
 
             Exception sessionLoadingException = null;
@@ -914,15 +960,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
          * based on method parameters.  This method is useful when JAXB is used as
          * the binding layer for a Web Service provider.
          */
-        private JAXBContextState createContextState(final Class[] classesToBeBound, Map<String, XmlBindings> xmlBindings) throws javax.xml.bind.JAXBException {
-            JaxbClassLoader loader = PrivilegedAccessHelper.shouldUsePrivilegedAccess()
-                    ? AccessController.doPrivileged(new PrivilegedAction<JaxbClassLoader>() {
-                        @Override
-                        public JaxbClassLoader run() {
-                            return new JaxbClassLoader(classLoader, classesToBeBound);
-                        }
-                    })
-                    : new JaxbClassLoader(classLoader, classesToBeBound);
+        private JAXBContextState createContextState(final JaxbClassLoader loader, Class<?>[] classesToBeBound, Map<String, XmlBindings> xmlBindings) throws javax.xml.bind.JAXBException {
             String defaultTargetNamespace = null;
             AnnotationHelper annotationHelper = null;
             boolean enableXmlAccessorFactory = false;
@@ -960,7 +998,7 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
                 jModel.setMetadataCompletePackageMap(metadataComplete);
             }
 
-            jModel.setHasXmlBindings(xmlBindings != null || !xmlBindings.isEmpty());
+            jModel.setHasXmlBindings(!xmlBindings.isEmpty());
             JavaModelInputImpl inputImpl = new JavaModelInputImpl(classesToBeBound, jModel);
             if (properties != null) enableFacetsIfPropertySetTrue(inputImpl, properties);
             try {
@@ -1083,6 +1121,25 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
             });
         }
 
+        // JaxbClassLoader creator to be used ONLY in reateContextState() method
+        // statement PrivilegedAccessHelper.callDoPrivileged
+        private static final class CreateJaxbClassLoader implements PrivilegedCallable<JaxbClassLoader> {
+
+            private final ClassLoader classLoader;
+            private TypeMappingInfo[] types;
+
+            private CreateJaxbClassLoader(final ClassLoader classLoader, final TypeMappingInfo[] types) {
+                this.classLoader = classLoader;
+                this.types = types;
+            }
+
+            @Override
+            public JaxbClassLoader call() {
+                return new JaxbClassLoader(classLoader, types);
+            }
+        }
+
+        @Override
         protected JAXBContextState createContextState() throws javax.xml.bind.JAXBException {
             // Check properties map for eclipselink-oxm.xml entries
             Map<String, XmlBindings> xmlBindings = JAXBContextFactory.getXmlBindingsFromProperties(properties, classLoader);
@@ -1110,15 +1167,8 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
                 typesToBeBound = getXmlBindingsClasses(entry.getValue(), classLoader, typesToBeBound);
             }
 
-            final TypeMappingInfo[] types = typesToBeBound;
-
-            JaxbClassLoader loader = PrivilegedAccessHelper.shouldUsePrivilegedAccess()
-                    ? AccessController.doPrivileged(new PrivilegedAction<JaxbClassLoader>() {
-                        public JaxbClassLoader run() {
-                            return new JaxbClassLoader(classLoader, types);
-                        }
-                    })
-                    : new JaxbClassLoader(classLoader, types);
+            final CreateJaxbClassLoader createJaxbClassLoader = new CreateJaxbClassLoader(classLoader, typesToBeBound);
+            final JaxbClassLoader loader = PrivilegedAccessHelper.callDoPrivileged(createJaxbClassLoader);
 
             JavaModelImpl jModel;
             if (annotationHelper != null) {
@@ -1527,8 +1577,8 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
             //Add any types that we didn't generate descriptors for (built in types)
             if (boundTypes != null) {
                 for (TypeMappingInfo next : this.boundTypes) {
-                    if (this.typeToSchemaType.get(next) == null) {
-                        Type nextType = next.getType();
+                    Type nextType = next.getType();
+                    if (this.typeToSchemaType.get(nextType) == null) {
                         QName name = getSchemaTypeForTypeMappingInfo(nextType);
                         if (name != null) {
                             this.typeToSchemaType.put(nextType, name);
@@ -1592,6 +1642,9 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
                 setPropertyOnMarshaller(JAXBContextProperties.BEAN_VALIDATION_NO_OPTIMISATION, marshaller);
                 setPropertyOnMarshaller(JAXBContextProperties.JSON_TYPE_COMPATIBILITY, marshaller);
                 setPropertyOnMarshaller(JAXBContextProperties.JSON_USE_XSD_TYPES_WITH_PREFIX, marshaller);
+                setPropertyOnMarshaller(JAXBContextProperties.JSON_TYPE_ATTRIBUTE_NAME, marshaller);
+                setPropertyOnMarshaller(JAXBContextProperties.MOXY_LOGGING_LEVEL, marshaller);
+                setPropertyOnMarshaller(JAXBContextProperties.MOXY_LOG_PAYLOAD, marshaller);
             }
 
             return marshaller;
@@ -1625,23 +1678,26 @@ public class JAXBContext extends javax.xml.bind.JAXBContext {
                 setPropertyOnUnmarshaller(JAXBContextProperties.BEAN_VALIDATION_NO_OPTIMISATION, unmarshaller);
                 setPropertyOnUnmarshaller(JAXBContextProperties.JSON_TYPE_COMPATIBILITY, unmarshaller);
                 setPropertyOnUnmarshaller(JAXBContextProperties.JSON_USE_XSD_TYPES_WITH_PREFIX, unmarshaller);
+                setPropertyOnUnmarshaller(JAXBContextProperties.JSON_TYPE_ATTRIBUTE_NAME, unmarshaller);
+                setPropertyOnUnmarshaller(JAXBContextProperties.MOXY_LOGGING_LEVEL, unmarshaller);
+                setPropertyOnUnmarshaller(JAXBContextProperties.MOXY_LOG_PAYLOAD, unmarshaller);
             }
             return unmarshaller;
         }
 
         public JAXBBinder createBinder(JAXBContext context) {
-		XMLMarshaller marshaller = null;
-		XMLUnmarshaller unmarshaller = null;
-		try {
-			marshaller = createMarshaller(context).getXMLMarshaller();
-			unmarshaller = createUnmarshaller(context).getXMLUnmarshaller();
-		} catch (javax.xml.bind.JAXBException e) {
-			// log something
-			marshaller = context.getXMLContext().createMarshaller();
-			unmarshaller = context.getXMLContext().createUnmarshaller();
-		}
+        XMLMarshaller marshaller = null;
+        XMLUnmarshaller unmarshaller = null;
+        try {
+            marshaller = createMarshaller(context).getXMLMarshaller();
+            unmarshaller = createUnmarshaller(context).getXMLUnmarshaller();
+        } catch (javax.xml.bind.JAXBException e) {
+            // log something
+            marshaller = context.getXMLContext().createMarshaller();
+            unmarshaller = context.getXMLContext().createUnmarshaller();
+        }
 
-		return new JAXBBinder(context, marshaller, unmarshaller);
+        return new JAXBBinder(context, marshaller, unmarshaller);
         }
 
         private void setPropertyOnMarshaller(String propertyName, JAXBMarshaller marshaller) throws PropertyException {
