@@ -21,9 +21,9 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
 
 /**
  * <p>This {@link JPQLGrammar} provides support for parsing JPQL queries defined in
- * <a href="http://jcp.org/en/jsr/detail?id=317">JSR-338 - Java Persistence 2.1</a>.</p>
+ * <a href="http://jcp.org/en/jsr/detail?id=317">JSR-337 - Java Persistence 2.0</a>.</p>
  *
- * The following is the JPQL grammar defined in JPA version 2.1.
+ * The following is the JPQL grammar defined in JPA version 2.0.
  *
  * <pre><code> QL_statement ::= select_statement | update_statement | delete_statement
  *
@@ -39,18 +39,14 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  *
  * range_variable_declaration ::= entity_name [AS] identification_variable
  *
- * join ::= join_spec join_association_path_expression [AS] identification_variable [join_condition]
+ * join ::= join_spec join_association_path_expression [AS] identification_variable
  *
- * fetch_join ::= join_spec FETCH join_association_path_expression [join_condition]
+ * fetch_join ::= join_spec FETCH join_association_path_expression
  *
  * join_spec ::= [ LEFT [OUTER] | INNER ] JOIN
  *
- * join_condition ::= ON conditional_expression
- *
  * join_association_path_expression ::= join_collection_valued_path_expression |
- *                                      join_single_valued_path_expression |
- *                                      TREAT(join_collection_valued_path_expression AS subtype) |
- *                                      TREAT(join_single_valued_path_expression AS subtype)
+ *                                      join_single_valued_path_expression
  *
  * join_collection_valued_path_expression ::= identification_variable.{single_valued_embeddable_object_field.}*collection_valued_field
  *
@@ -58,36 +54,27 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  *
  * collection_member_declaration ::= IN (collection_valued_path_expression) [AS] identification_variable
  *
- * qualified_identification_variable ::= composable_qualified_identification_variable |
+ * qualified_identification_variable ::= KEY(identification_variable) |
+ *                                       VALUE(identification_variable) |
  *                                       ENTRY(identification_variable)
  *
- * composable_qualified_identification_variable ::= KEY(identification_variable) |
- *                                                  VALUE(identification_variable)
- *
  * single_valued_path_expression ::= qualified_identification_variable |
- *                                   TREAT(qualified_identification_variable AS subtype) |
  *                                   state_field_path_expression |
  *                                   single_valued_object_path_expression
  *
  * general_identification_variable ::= identification_variable |
- *                                     composable_qualified_identification_variable
+ *                                     KEY(identification_variable) |
+ *                                     VALUE(identification_variable)
  *
- * general_subpath ::= simple_subpath | treated_subpath{.single_valued_object_field}*
+ * state_field_path_expression ::= general_identification_variable.{single_valued_object_field.}*state_field
  *
- * simple_subpath ::= general_identification_variable |
- *                    general_identification_variable{.single_valued_object_field}*
+ * single_valued_object_path_expression ::= general_identification_variable.{single_valued_object_field.}*single_valued_object_field
  *
- * treated_subpath ::= TREAT(general_subpath AS subtype)
- *
- * state_field_path_expression ::= general_subpath.state_field
- *
- * single_valued_object_path_expression ::= general_subpath.single_valued_object_field
- *
- * collection_valued_path_expression ::= general_subpath.{collection_valued_field}
+ * collection_valued_path_expression ::= general_identification_variable.{single_valued_object_field.}*collection_valued_field
  *
  * update_clause ::= UPDATE entity_name [[AS] identification_variable] SET update_item {, update_item}*
  *
- * update_item ::= [identification_variable.]{single_valued_embeddable_object_field.}*{state_field | single_valued_object_field} = new_value
+ * update_item ::= [identification_variable.]{state_field | single_valued_object_field} = new_value
  *
  * new_value ::= scalar_expression |
  *               simple_entity_expression |
@@ -116,8 +103,7 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  * aggregate_expression ::= { AVG | MAX | MIN | SUM } ([DISTINCT] state_field_path_expression) |
  *                          COUNT ([DISTINCT] identification_variable |
  *                                            state_field_path_expression |
- *                                            single_valued_object_path_expression) |
- *                          function_invocation
+ *                                            single_valued_object_path_expression)
  *
  * where_clause ::= WHERE conditional_expression
  *
@@ -140,30 +126,23 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  *                                                   derived_path_expression [AS] identification_variable {join}* |
  *                                                   derived_collection_member_declaration
  *
- * derived_path_expression ::= general_derived_path.single_valued_object_field |
- *                             general_derived_path.collection_valued_field
- *
- * general_derived_path ::= simple_derived_path |
- *                          treated_derived_path{.single_valued_object_field}*
- *
- * simple_derived_path ::= superquery_identification_variable{.single_valued_object_field}*
- *
- * treated_derived_path ::= TREAT(general_derived_path AS subtype)
+ * derived_path_expression ::= superquery_identification_variable.{single_valued_object_field.}*collection_valued_field |
+ *                             superquery_identification_variable.{single_valued_object_field.}*single_valued_object_field
  *
  * derived_collection_member_declaration ::= IN superquery_identification_variable.{single_valued_object_field.}*collection_valued_field
  *
  * simple_select_clause ::= SELECT [DISTINCT] simple_select_expression
  *
- * simple_select_expression::= single_valued_path_expression |
- *                             scalar_expression |
- *                             aggregate_expression |
- *                             identification_variable
+ * simple_select_expression ::= single_valued_path_expression |
+ *                              scalar_expression |
+ *                              aggregate_expression |
+ *                              identification_variable
  *
- * scalar_expression ::= arithmetic_expression |
- *                       string_expression |
- *                       enum_expression |
- *                       datetime_expression |
- *                       boolean_expression |
+ * scalar_expression ::= simple_arithmetic_expression |
+ *                       string_primary |
+ *                       enum_primary |
+ *                       datetime_primary |
+ *                       boolean_primary |
  *                       case_expression |
  *                       entity_type_expression
  *
@@ -224,7 +203,9 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  *
  * comparison_operator ::= = | {@literal > | >= | < | <= | <>}
  *
- * arithmetic_expression ::= arithmetic_term | arithmetic_expression { + | - } arithmetic_term
+ * arithmetic_expression ::= simple_arithmetic_expression | (subquery)
+ *
+ * simple_arithmetic_expression ::= arithmetic_term | simple_arithmetic_expression { + | - } arithmetic_term
  *
  * arithmetic_term ::= arithmetic_factor | arithmetic_term { * | / } arithmetic_factor
  *
@@ -232,44 +213,43 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  *
  * arithmetic_primary ::= state_field_path_expression |
  *                        numeric_literal |
- *                        (arithmetic_expression) |
+ *                        (simple_arithmetic_expression) |
  *                        input_parameter |
  *                        functions_returning_numerics |
  *                        aggregate_expression |
- *                        case_expression |
- *                        function_invocation |
- *                        (subquery)
+ *                        case_expression
  *
- * string_expression ::= state_field_path_expression |
- *                       string_literal |
- *                       input_parameter |
- *                       functions_returning_strings |
- *                       aggregate_expression |
- *                       case_expression |
- *                       function_invocation |
- *                       (subquery)
+ * string_expression ::= string_primary | (subquery)
  *
- * datetime_expression ::= state_field_path_expression |
- *                         input_parameter |
- *                         functions_returning_datetime |
- *                         aggregate_expression |
- *                         case_expression |
- *                         function_invocation |
- *                         date_time_timestamp_literal |
- *                         (subquery)
+ * string_primary ::= state_field_path_expression |
+ *                    string_literal |
+ *                    input_parameter |
+ *                    functions_returning_strings |
+ *                    aggregate_expression |
+ *                    case_expression
  *
- * boolean_expression ::= state_field_path_expression |
- *                        boolean_literal |
- *                        input_parameter |
- *                        case_expression |
- *                        function_invocation |
- *                        (subquery)
+ * datetime_primary ::= state_field_path_expression |
+ *                      input_parameter |
+ *                      functions_returning_datetime |
+ *                      aggregate_expression |
+ *                      case_expression |
+ *                      date_time_timestamp_literal
  *
- * enum_expression ::= state_field_path_expression |
- *                     enum_literal |
+ * datetime_expression ::= datetime_primary | (subquery)
+ *
+ * boolean_expression ::= boolean_primary | (subquery)
+ *
+ * boolean_primary ::= state_field_path_expression |
+ *                     boolean_literal |
  *                     input_parameter |
- *                     case_expression |
- *                     (subquery)
+ *                     case_expression
+ *
+ * enum_expression ::= enum_primary | (subquery)
+ *
+ * enum_primary ::= state_field_path_expression |
+ *                  enum_literal |
+ *                  input_parameter |
+ *                  case_expression
  *
  * entity_expression ::= single_valued_object_path_expression | simple_entity_expression
  *
@@ -282,30 +262,25 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  *
  * type_discriminator ::= TYPE(identification_variable | single_valued_object_path_expression | input_parameter)
  *
- * functions_returning_numerics ::= LENGTH(string_expression) |
- *                                  LOCATE(string_expression, string_expression[, arithmetic_expression]) |
- *                                  ABS(arithmetic_expression) |
- *                                  SQRT(arithmetic_expression) |
- *                                  MOD(arithmetic_expression, arithmetic_expression) |
+ * functions_returning_numerics ::= LENGTH(string_primary) |
+ *                                  LOCATE(string_primary, string_primary[, simple_arithmetic_expression]) |
+ *                                  ABS(simple_arithmetic_expression) |
+ *                                  SQRT(simple_arithmetic_expression) |
+ *                                  MOD(simple_arithmetic_expression, simple_arithmetic_expression) |
  *                                  SIZE(collection_valued_path_expression) |
  *                                  INDEX(identification_variable)
  *
- * functions_returning_datetime ::= CURRENT_DATE | CURRENT_TIME | CURRENT_TIMESTAMP
+ * functions_returning_datetime ::= CURRENT_DATE | CURRENT_TIME | CURRENT_TIMESTAMP | literal_temporal
  *
- * functions_returning_strings ::= CONCAT(string_expression, string_expression {, string_expression}*) |
- *                                 SUBSTRING(string_expression, arithmetic_expression [, arithmetic_expression]) |
- *                                 TRIM([[trim_specification] [trim_character] FROM] string_expression) |
- *                                 LOWER(string_expression) |
- *                                 UPPER(string_expression)
+ * functions_returning_strings ::= CONCAT(string_primary, string_primary {, string_primary}*) |
+ *                                 SUBSTRING(string_primary, simple_arithmetic_expression [, simple_arithmetic_expression]) |
+ *                                 TRIM([[trim_specification] [trim_character] FROM] string_primary) |
+ *                                 LOWER(string_primary) |
+ *                                 UPPER(string_primary)
  *
  * trim_specification ::= LEADING | TRAILING | BOTH
  *
- * function_invocation ::= FUNCTION(string_literal {, function_arg}*)
- *
- * function_arg ::= literal |
- *                  state_field_path_expression |
- *                  input_parameter |
- *                  scalar_expression
+ * trim_character ::= string_literal | input_parameter
  *
  * case_expression ::= general_case_expression |
  *                     simple_case_expression |
@@ -358,32 +333,32 @@ import static org.eclipse.persistence.jpa.jpql.parser.Expression.*;
  * to solicit feedback from pioneering adopters on the understanding that any code that uses this
  * API will almost certainly be broken (repeatedly) as the API evolves.</p>
  *
- * @version 2.5
+ * @version 2.5.1
  * @since 2.4
  * @author Pascal Filion
  */
 @SuppressWarnings("nls")
-public final class JPQLGrammar2_1 extends AbstractJPQLGrammar {
+public final class JPQLGrammar2_0 extends AbstractJPQLGrammar {
 
     /**
-     * The singleton instance of this {@link JPQLGrammar2_1}.
+     * The singleton instance of this {@link JPQLGrammar2_0}.
      */
-    private static final JPQLGrammar INSTANCE = new JPQLGrammar2_1();
+    private static final JPQLGrammar INSTANCE = new JPQLGrammar2_0();
 
     /**
-     * Creates a new <code>JPQLExtension2_1</code>.
+     * Creates a new <code>JPQLExtension2_0</code>.
      */
-    public JPQLGrammar2_1() {
+    public JPQLGrammar2_0() {
         super();
     }
 
     /**
-     * Creates a new <code>JPQLGrammar2_1</code>.
+     * Creates a new <code>JPQLGrammar2_0</code>.
      *
      * @param jpqlGrammar The {@link JPQLGrammar} to extend with the content of this one without
      * instantiating the base {@link JPQLGrammar}
      */
-    private JPQLGrammar2_1(AbstractJPQLGrammar jpqlGrammar) {
+    private JPQLGrammar2_0(AbstractJPQLGrammar jpqlGrammar) {
         super(jpqlGrammar);
     }
 
@@ -395,14 +370,14 @@ public final class JPQLGrammar2_1 extends AbstractJPQLGrammar {
      * instantiating the base {@link JPQLGrammar}
      */
     public static void extend(AbstractJPQLGrammar jpqlGrammar) {
-        new JPQLGrammar2_1(jpqlGrammar);
+        new JPQLGrammar2_0(jpqlGrammar);
     }
 
     /**
      * Returns the singleton instance of the default implementation of {@link JPQLGrammar} which
-     * provides support for the JPQL grammar defined in the JPA 2.1 functional specification.
+     * provides support for the JPQL grammar defined in the JPA 2.0 functional specification.
      *
-     * @return The {@link JPQLGrammar} that only has support for JPA 2.1
+     * @return The {@link JPQLGrammar} that only has support for JPA 2.0
      */
     public static JPQLGrammar instance() {
         return INSTANCE;
@@ -413,14 +388,14 @@ public final class JPQLGrammar2_1 extends AbstractJPQLGrammar {
      */
     @Override
     protected JPQLGrammar buildBaseGrammar() {
-        return new JPQLGrammar2_0();
+        return new JPQLGrammar1_0();
     }
 
     /**
      * {@inheritDoc}
      */
     public JPAVersion getJPAVersion() {
-        return JPAVersion.VERSION_2_1;
+        return JPAVersion.VERSION_2_0;
     }
 
     /**
@@ -443,54 +418,42 @@ public final class JPQLGrammar2_1 extends AbstractJPQLGrammar {
     @Override
     protected void initializeBNFs() {
 
-        registerBNF(new FunctionExpressionBNF());
-        registerBNF(new FunctionItemBNF());
-        registerBNF(new OnClauseBNF());
-        registerBNF(new TreatExpressionBNF());
+        registerBNF(new CaseExpressionBNF());
+        registerBNF(new CaseOperandBNF());
+        registerBNF(new CoalesceExpressionBNF());
+        registerBNF(new ElseExpressionBNF());
+        registerBNF(new EntityTypeExpressionBNF());
+        registerBNF(new EntityTypeLiteralBNF());
+        registerBNF(new GeneralCaseExpressionBNF());
+        registerBNF(new InternalCoalesceExpressionBNF());
+        registerBNF(new InternalEntityTypeExpressionBNF());
+        registerBNF(new InternalWhenClauseBNF());
+        registerBNF(new NullIfExpressionBNF());
+        registerBNF(new QualifiedIdentificationVariableBNF());
+        registerBNF(new ResultVariableBNF());
+        registerBNF(new SingleValuedObjectPathExpressionBNF());
+        registerBNF(new TypeExpressionBNF());
+        registerBNF(new WhenClauseBNF());
 
-        // Extend the query BNF to add support for TREAT
-        addChildBNF(CollectionValuedPathExpressionBNF.ID, TreatExpressionBNF.ID);
-        addChildBNF(JoinAssociationPathExpressionBNF.ID,  TreatExpressionBNF.ID);
-        addChildBNF(SingleValuedPathExpressionBNF.ID,     TreatExpressionBNF.ID);
-        addChildBNF(StateFieldPathExpressionBNF.ID,       TreatExpressionBNF.ID);
-
-        // Extends the query BNF to add support for FUNCTION
-        addChildBNF(AggregateExpressionBNF.ID, FunctionExpressionBNF.ID);
-
-        // string_primary becomes string_expression in CONCAT
-        addChildBNF(InternalConcatExpressionBNF.ID, StringExpressionBNF.ID);
-
-        // string_primary becomes string_expression, simply add the new query BNFs
-        addChildBNF(StringPrimaryBNF.ID,    FunctionExpressionBNF.ID);
-        addChildBNF(StringPrimaryBNF.ID,    SubqueryBNF.ID);
-        addChildBNF(StringExpressionBNF.ID, FunctionExpressionBNF.ID);
-        addChildBNF(StringExpressionBNF.ID, SubqueryBNF.ID);
-
-        // datetime_primary becomes datetime_expression, simply add the new query BNFs
-        addChildBNF(DateTimePrimaryBNF.ID, FunctionExpressionBNF.ID);
-        addChildBNF(DateTimePrimaryBNF.ID, SubqueryBNF.ID);
-
-        // boolean_primary becomes boolean_expression, simply add the new query BNFs
-        addChildBNF(BooleanPrimaryBNF.ID, FunctionExpressionBNF.ID);
-        addChildBNF(BooleanPrimaryBNF.ID, SubqueryBNF.ID);
-
-        // enum_primary becomes enum_expression, simply add the new query BNFs
-        addChildBNF(EnumLiteralBNF.ID, SubqueryBNF.ID);
-
-        // arithmetic_primary becomes arithmetic_expression, simply add the new query BNFs
-        addChildBNF(ArithmeticPrimaryBNF.ID, FunctionExpressionBNF.ID);
-        addChildBNF(ArithmeticPrimaryBNF.ID, SubqueryBNF.ID);
-
-        // datetime_expression
-        addChildBNF(DatetimeExpressionBNF.ID, FunctionExpressionBNF.ID);
-        addChildBNF(DatetimeExpressionBNF.ID, SubqueryBNF.ID);
-
-        // boolean_expression
-        addChildBNF(BooleanExpressionBNF.ID, FunctionExpressionBNF.ID);
-        addChildBNF(BooleanExpressionBNF.ID, SubqueryBNF.ID);
-
-        // enum_expression
-        addChildBNF(EnumExpressionBNF.ID, SubqueryBNF.ID);
+        // Extend some query BNFs
+        addChildBNF(ArithmeticPrimaryBNF.ID,              CaseExpressionBNF.ID);
+        addChildBNF(BooleanPrimaryBNF.ID,                 CaseExpressionBNF.ID);
+        addChildBNF(CollectionValuedPathExpressionBNF.ID, GeneralIdentificationVariableBNF.ID);
+        addChildBNF(ComparisonExpressionBNF.ID,           EntityTypeExpressionBNF.ID);
+        addChildBNF(DateTimePrimaryBNF.ID,                CaseExpressionBNF.ID);
+        addChildBNF(DateTimePrimaryBNF.ID,                DateTimeTimestampLiteralBNF.ID);
+        addChildBNF(EntityExpressionBNF.ID,               SingleValuedObjectPathExpressionBNF.ID);
+        addChildBNF(EnumPrimaryBNF.ID,                    CaseExpressionBNF.ID);
+        addChildBNF(InExpressionExpressionBNF.ID,         TypeExpressionBNF.ID);
+        addChildBNF(InternalCountBNF.ID,                  SingleValuedObjectPathExpressionBNF.ID);
+        addChildBNF(LiteralBNF.ID,                        EntityTypeLiteralBNF.ID);
+        addChildBNF(ScalarExpressionBNF.ID,               CaseExpressionBNF.ID);
+        addChildBNF(ScalarExpressionBNF.ID,               EntityTypeExpressionBNF.ID);
+        addChildBNF(SelectExpressionBNF.ID,               ScalarExpressionBNF.ID);
+        addChildBNF(SingleValuedPathExpressionBNF.ID,     QualifiedIdentificationVariableBNF.ID);
+        addChildBNF(SingleValuedPathExpressionBNF.ID,     SingleValuedObjectPathExpressionBNF.ID);
+        addChildBNF(StateFieldPathExpressionBNF.ID,       GeneralIdentificationVariableBNF.ID);
+        addChildBNF(StringPrimaryBNF.ID,                  CaseExpressionBNF.ID);
     }
 
     /**
@@ -499,9 +462,25 @@ public final class JPQLGrammar2_1 extends AbstractJPQLGrammar {
     @Override
     protected void initializeExpressionFactories() {
 
-        registerFactory(new FunctionExpressionFactory(FunctionExpressionFactory.ID, FUNCTION));
-        registerFactory(new OnClauseFactory());
-        registerFactory(new TreatExpressionFactory());
+        registerFactory(new CaseExpressionFactory());
+        registerFactory(new CoalesceExpressionFactory());
+        registerFactory(new EntityTypeLiteralFactory());
+        registerFactory(new EntryExpressionFactory());
+        registerFactory(new KeyExpressionFactory());
+        registerFactory(new IndexExpressionFactory());
+        registerFactory(new NullIfExpressionFactory());
+        registerFactory(new ResultVariableFactory());
+        registerFactory(new TypeExpressionFactory());
+        registerFactory(new ValueExpressionFactory());
+        registerFactory(new WhenClauseFactory());
+        registerFactory(new EntityTypeLiteralFactory());
+
+        // Extend INDEX expression
+        addChildFactory(FunctionsReturningNumericsBNF.ID, IndexExpressionFactory.ID);
+
+        // Add support for entity type literal
+        addChildBNF(InExpressionItemBNF.ID, EntityTypeLiteralBNF.ID);
+        setFallbackExpressionFactoryId(InExpressionItemBNF.ID, EntityTypeLiteralFactory.ID);
     }
 
     /**
@@ -510,13 +489,31 @@ public final class JPQLGrammar2_1 extends AbstractJPQLGrammar {
     @Override
     protected void initializeIdentifiers() {
 
-        registerIdentifierRole(FUNCTION,    IdentifierRole.FUNCTION); // FUNCTION(n, x1, ..., x2)
-        registerIdentifierRole(ON,          IdentifierRole.CLAUSE);   // ON x
-        registerIdentifierRole(TREAT,       IdentifierRole.FUNCTION); // TREAT(x AS y)
+        registerIdentifierRole(CASE,        IdentifierRole.FUNCTION);           // CASE {o} {WHEN x THEN y}* ELSE z END
+        registerIdentifierRole(COALESCE,    IdentifierRole.FUNCTION);           // COALLESCE(x {, y}+)
+        registerIdentifierRole(ELSE,        IdentifierRole.COMPOUND_FUNCTION);  // Part of CASE WHEN ELSE END
+        registerIdentifierRole(END,         IdentifierRole.COMPLEMENT);       // Part of CASE WHEN ELSE END
+        registerIdentifierRole(ENTRY,       IdentifierRole.FUNCTION);           // ENTRY(x)
+        registerIdentifierRole(INDEX,       IdentifierRole.FUNCTION);           // INDEX(x)
+        registerIdentifierRole(KEY,         IdentifierRole.FUNCTION);           // KEY(x)
+        registerIdentifierRole(NULLIF,      IdentifierRole.FUNCTION);           // NULLIF(x, y)
+        registerIdentifierRole(THEN,        IdentifierRole.COMPOUND_FUNCTION);  // Part of CASE WHEN ELSE END
+        registerIdentifierRole(TYPE,        IdentifierRole.FUNCTION);           // TYPE(x)
+        registerIdentifierRole(VALUE,       IdentifierRole.FUNCTION);           // VALUE(x)
+        registerIdentifierRole(WHEN,        IdentifierRole.COMPOUND_FUNCTION);  // Part of CASE WHEN ELSE END
 
-        registerIdentifierVersion(FUNCTION, JPAVersion.VERSION_2_1);
-        registerIdentifierVersion(ON,       JPAVersion.VERSION_2_1);
-        registerIdentifierVersion(TREAT,    JPAVersion.VERSION_2_1);
+        registerIdentifierVersion(CASE,     JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(COALESCE, JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(ELSE,     JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(END,      JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(ENTRY,    JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(INDEX,    JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(KEY,      JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(NULLIF,   JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(THEN,     JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(TYPE,     JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(VALUE,    JPAVersion.VERSION_2_0);
+        registerIdentifierVersion(WHEN,     JPAVersion.VERSION_2_0);
     }
 
     /**
@@ -524,6 +521,6 @@ public final class JPQLGrammar2_1 extends AbstractJPQLGrammar {
      */
     @Override
     public String toString() {
-        return "JPQLGrammar 2.1";
+        return "JPQLGrammar 2.0";
     }
 }
