@@ -1,19 +1,62 @@
-/*******************************************************************************
- * Copyright (c) 2011, 2016 Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 2011, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- * 	   dclarke/tware - initial
- *     2014-09-01-2.6.0 Dmitry Kornilov
- *       - JPARS 2.0 related changes
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//        dclarke/tware - initial
+//     2014-09-01-2.6.0 Dmitry Kornilov
+//       - JPARS 2.0 related changes
 package org.eclipse.persistence.jpa.rs;
 
+import java.beans.BeanInfo;
+import java.beans.IntrospectionException;
+import java.beans.Introspector;
+import java.beans.PropertyDescriptor;
+import java.io.BufferedInputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.Field;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.net.URI;
+import java.security.AccessController;
+import java.security.PrivilegedActionException;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+
+import javax.persistence.EntityManager;
+import javax.persistence.EntityManagerFactory;
+import javax.persistence.Query;
+import javax.persistence.RollbackException;
+import javax.persistence.spi.PersistenceUnitInfo;
+import javax.ws.rs.core.MediaType;
+import javax.xml.bind.JAXBElement;
+import javax.xml.bind.JAXBException;
+import javax.xml.bind.Marshaller;
+import javax.xml.bind.Unmarshaller;
+import javax.xml.bind.ValidationEvent;
+import javax.xml.bind.ValidationEventHandler;
+import javax.xml.bind.annotation.adapters.XmlAdapter;
+import javax.xml.stream.XMLOutputFactory;
+import javax.xml.stream.XMLStreamWriter;
+import javax.xml.transform.stream.StreamSource;
+
 import org.eclipse.persistence.config.PersistenceUnitProperties;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.descriptors.FetchGroupManager;
@@ -75,53 +118,16 @@ import org.eclipse.persistence.sessions.DatabaseSession;
 import org.eclipse.persistence.sessions.Session;
 import org.eclipse.persistence.sessions.UnitOfWork;
 
-import javax.persistence.EntityManager;
-import javax.persistence.EntityManagerFactory;
-import javax.persistence.Query;
-import javax.persistence.RollbackException;
-import javax.persistence.spi.PersistenceUnitInfo;
-import javax.ws.rs.core.MediaType;
-import javax.xml.bind.JAXBElement;
-import javax.xml.bind.JAXBException;
-import javax.xml.bind.Marshaller;
-import javax.xml.bind.Unmarshaller;
-import javax.xml.bind.ValidationEvent;
-import javax.xml.bind.ValidationEventHandler;
-import javax.xml.bind.annotation.adapters.XmlAdapter;
-import javax.xml.stream.XMLOutputFactory;
-import javax.xml.stream.XMLStreamWriter;
-import javax.xml.transform.stream.StreamSource;
-import java.beans.BeanInfo;
-import java.beans.Introspector;
-import java.beans.PropertyDescriptor;
-import java.io.BufferedInputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.lang.reflect.Constructor;
-import java.lang.reflect.Field;
-import java.lang.reflect.Method;
-import java.net.URI;
-import java.security.AccessController;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-
 /**
  * A wrapper around the JPA and JAXB artifacts used to persist an application.
- * 
+ *
  * A PersistenceContext provides the capability of using the same persistence unit in JPA to
- * to interact with a Database or other JPA-capable data source and in JAXB to interact with either 
+ * to interact with a Database or other JPA-capable data source and in JAXB to interact with either
  * XML or JSON.
- * 
- * A PersistenceContext can wrap either an existing persistence unit (EntityManagerFactory), or it can be used to bootstrap a 
+ *
+ * A PersistenceContext can wrap either an existing persistence unit (EntityManagerFactory), or it can be used to bootstrap a
  * fully dynamic persistence unit.
- * 
+ *
  * @author douglas.clarke, tom.ware
  */
 public class PersistenceContext {
@@ -254,7 +260,7 @@ public class PersistenceContext {
 
     /**
      * This method is used to help construct a JAXBContext from an existing EntityManagerFactory.
-     * 
+     *
      * For each package in the EntityManagerFactory, a MetadataSource that is capable of building a JAXBContext
      * that creates the same mappings in JAXB is created.  These MetadataSources are used to constuct the JAXContext
      * that is used for JSON and XML translation.
@@ -263,7 +269,7 @@ public class PersistenceContext {
      */
     protected void addDynamicXMLMetadataSources(List<Object> metadataSources, AbstractSession session) {
         Set<String> packages = new HashSet<String>();
-        for (Class descriptorClass : session.getDescriptors().keySet()) {
+        for (Class<?> descriptorClass : session.getDescriptors().keySet()) {
             String packageName = "";
             int lastDotIndex = descriptorClass.getName().lastIndexOf('.');
             if (lastDotIndex > 0) {
@@ -284,7 +290,7 @@ public class PersistenceContext {
      * Persist an entity in JPA and commit.
      * @param tenantId
      * @param entity
-     * @throws Exception 
+     * @throws Exception
      */
     public void create(Map<String, String> tenantId, Object entity) throws Exception {
         EntityManager em = getEmf().createEntityManager(tenantId);
@@ -433,10 +439,9 @@ public class PersistenceContext {
      * Finalize.
      */
     @Override
-    public void finalize() {
-        this.emf.close();
-        this.emf = null;
-        this.jaxbContext = null;
+    protected void finalize() throws Throwable {
+        emf.close();
+        super.finalize();
     }
 
     /**
@@ -548,7 +553,7 @@ public class PersistenceContext {
      * @param entity
      * @param partner the partner
      * @return the object
-     *  
+     *
      */
     @SuppressWarnings({ "unchecked", "rawtypes" })
     public Object removeAttribute(Map<String, String> tenantId, String entityName, Object id, String attribute, String listItemId, Object entity, String partner)
@@ -583,7 +588,7 @@ public class PersistenceContext {
                     fieldName = field.getName();
                     if (fieldName.equals(attribute)) {
                         try {
-                            // call clear on this collection 
+                            // call clear on this collection
                             Object attributeValue = getAttribute(entity, attribute);
                             originalAttributeValue = attributeValue;
                             if (attributeValue instanceof Collection) {
@@ -638,7 +643,7 @@ public class PersistenceContext {
                     return value;
                 }
             }
-        } catch (Exception ex) {
+        } catch (IntrospectionException | PrivilegedActionException | IllegalAccessException | InvocationTargetException ex) {
             return null;
         }
         return null;
@@ -915,15 +920,16 @@ public class PersistenceContext {
         Query query = em.createNamedQuery(name);
         DatabaseQuery dbQuery = ((EJBQueryImpl<?>) query).getDatabaseQuery();
         if (parameters != null) {
-            Iterator i = parameters.keySet().iterator();
+            Iterator<?> i = parameters.entrySet().iterator();
             while (i.hasNext()) {
-                String key = (String) i.next();
+                Map.Entry<?, ?> entry = (Map.Entry<?, ?>) i.next();
+                String key = (String) entry.getKey();
                 Class parameterClass = null;
                 int index = dbQuery.getArguments().indexOf(key);
                 if (index >= 0) {
                     parameterClass = dbQuery.getArgumentTypes().get(index);
                 }
-                Object parameter = parameters.get(key);
+                Object parameter = entry.getValue();
                 if (parameterClass != null) {
                     parameter = ConversionManager.getDefaultManager().convertObject(parameter, parameterClass);
                 }
@@ -931,8 +937,8 @@ public class PersistenceContext {
             }
         }
         if (hints != null) {
-            for (String key : hints.keySet()) {
-                query.setHint(key, hints.get(key));
+            for (Map.Entry<String, ?> entry : hints.entrySet()) {
+                query.setHint(entry.getKey(), entry.getValue());
             }
         }
         return query;
@@ -953,15 +959,16 @@ public class PersistenceContext {
         Query query = em.createNamedQuery(name);
         DatabaseQuery dbQuery = ((EJBQueryImpl<?>) query).getDatabaseQuery();
         if (parameters != null) {
-            Iterator i = parameters.keySet().iterator();
+            Iterator<?> i = parameters.entrySet().iterator();
             while (i.hasNext()) {
-                String key = (String) i.next();
+                Map.Entry<?, ?> entry = (Map.Entry<?, ?>) i.next();
+                String key = (String) entry.getKey();
                 Class parameterClass = null;
                 int index = dbQuery.getArguments().indexOf(key);
                 if (index >= 0) {
                     parameterClass = dbQuery.getArgumentTypes().get(index);
                 }
-                Object parameter = parameters.get(key);
+                Object parameter = entry.getValue();
                 if (parameterClass != null) {
                     parameter = ConversionManager.getDefaultManager().convertObject(parameter, parameterClass);
                 }
@@ -969,8 +976,8 @@ public class PersistenceContext {
             }
         }
         if (hints != null) {
-            for (String key : hints.keySet()) {
-                query.setHint(key, hints.get(key));
+            for (Map.Entry<String, ?> entry : hints.entrySet()) {
+                query.setHint(entry.getKey(), entry.getValue());
             }
         }
         return query;
@@ -1007,6 +1014,7 @@ public class PersistenceContext {
      *
      * @return the string
      */
+    @Override
     public String toString() {
         return "PersistenceContext(name:" + getName() + ", version:" + getVersion() + ", identityHashCode:" + System.identityHashCode(this) + ")";
     }
@@ -1024,7 +1032,7 @@ public class PersistenceContext {
         if (JPARSLogger.isLoggableFinest(getSessionLog())) {
             in = in.markSupported() ? in : new BufferedInputStream(in);
             // TODO: Make readlimit configurable. Some http servers allow http post size to be unlimited.
-            // If this is the case and if an application is sending huge post requests while jpars log 
+            // If this is the case and if an application is sending huge post requests while jpars log
             // level configured to finest, this readlimit might not be sufficient.
             in.mark(52428800); // (~50MB)
             JPARSLogger.entering(getSessionLog(), CLASS_NAME, "unmarshalEntity", in);
@@ -1052,11 +1060,11 @@ public class PersistenceContext {
         unmarshaller.setEventHandler(new ValidationEventHandler() {
             @Override
             /*
-             * ReferenceAdaptor unmarshal throws exception if the object referred by a link 
-             * doesn't exist, and this handler is required to interrupt the unmarshal 
+             * ReferenceAdaptor unmarshal throws exception if the object referred by a link
+             * doesn't exist, and this handler is required to interrupt the unmarshal
              * operation under this condition.
-             * (non-Javadoc) @see javax.xml.bind.ValidationEventHandler#handleEvent(javax.xml.bind.ValidationEvent) 
-             * 
+             * (non-Javadoc) @see javax.xml.bind.ValidationEventHandler#handleEvent(javax.xml.bind.ValidationEvent)
+             *
              */
             public boolean handleEvent(ValidationEvent event) {
                 if (event.getSeverity() != ValidationEvent.WARNING) {
@@ -1076,10 +1084,10 @@ public class PersistenceContext {
 
         if (acceptedMediaType == MediaType.APPLICATION_JSON_TYPE) {
             // Part of the fix for https://bugs.eclipse.org/bugs/show_bug.cgi?id=394059
-            // This issue happens when request has objects derived from an abstract class. 
+            // This issue happens when request has objects derived from an abstract class.
             // JSON_INCLUDE_ROOT is set to false for  JPA-RS. This means JSON requests won't have root tag.
-            // The unmarshal method needs to be called with type, so that moxy can unmarshal the message based on type. 
-            // For xml, root tag is always set, unmarshaller must use root of the message for unmarshalling and type should 
+            // The unmarshal method needs to be called with type, so that moxy can unmarshal the message based on type.
+            // For xml, root tag is always set, unmarshaller must use root of the message for unmarshalling and type should
             // not be passed to unmarshal for xml type requests.
             JAXBElement<?> element = unmarshaller.unmarshal(new StreamSource(in), type);
             if (element.getValue() instanceof List<?>) {
@@ -1107,7 +1115,7 @@ public class PersistenceContext {
 
     /**
      * Make adjustments to an unmarshalled entity based on what is found in the weaved fields
-     * 
+     *
      * @param entity
      * @return
      */
@@ -1402,7 +1410,7 @@ public class PersistenceContext {
                     }
                 }
             }
-        } catch (Exception ex) {
+        } catch (RuntimeException | ReflectiveOperationException ex) {
             ex.printStackTrace();
             throw JPARSException.exceptionOccurred(ex);
         }
