@@ -1,41 +1,37 @@
-/*******************************************************************************
- * Copyright (c) 2011, 2014 Jenzabar, Inc, Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 2011, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2022 IBM Corporation. All rights reserved.
+ * Copyright (c) 2011, 2022 Jenzabar, Inc. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Jenzabar - Initial implementation
- ******************************************************************************/  
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Jenzabar - Initial implementation
 
 package org.eclipse.persistence.platform.database;
 
 import java.io.IOException;
 import java.io.Serializable; // for javadoc only
 import java.io.Writer;
-
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Types;
-
 import java.util.Collection;
 
-import org.eclipse.persistence.internal.databaseaccess.DatasourcePlatform; // for javadoc only
-
-import org.eclipse.persistence.internal.sessions.AbstractSession;
-
 import org.eclipse.persistence.expressions.ExpressionOperator;
-
+import org.eclipse.persistence.internal.databaseaccess.DatasourcePlatform; // for javadoc only
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.helper.DatabaseTable;
-
-import org.eclipse.persistence.platform.database.H2Platform; // for javadoc only
-import org.eclipse.persistence.platform.database.InformixPlatform;
-
+import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.queries.UpdateAllQuery; // for javadoc only
+// for javadoc only
 
 /**
  * An {@link InformixPlatform} that fixes many EclipseLink bugs
@@ -51,7 +47,7 @@ import org.eclipse.persistence.queries.UpdateAllQuery; // for javadoc only
  *
  * @see <a
  * href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=401746">EclipseLink
- * bug 401746</a> 
+ * bug 401746</a>
  *
  * @see <a
  * href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=402037">EclipseLink
@@ -126,12 +122,10 @@ public class Informix11Platform extends InformixPlatform {
    */
   @Override
   protected void initializePlatformOperators() {
-    final ExpressionOperator distinctOverride = ExpressionOperator.getOperator(Integer.valueOf(ExpressionOperator.Distinct));
-    assert distinctOverride != null;
-    distinctOverride.printsAs("DISTINCT "); // no parens, one space
     super.initializePlatformOperators();
     this.addOperator(this.currentDateOperator());
     this.addOperator(this.currentTimeOperator());
+    this.addOperator(this.distinctOperator());
   }
 
   /**
@@ -172,6 +166,25 @@ public class Informix11Platform extends InformixPlatform {
     return ExpressionOperator.simpleFunctionNoParentheses(ExpressionOperator.CurrentDate, "CURRENT YEAR TO FRACTION(3)");
   }
 
+  /**
+   * Fixes <a
+   * href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=402600">EclipseLink
+   * bug 402600</a> by making sure that the {@link
+   * ExpressionOperator#distinct() Distinct} {@link
+   * ExpressionOperator} is set to {@linkplain
+   * ExpressionOperator#printsAs(String) print as} {@code
+   * DISTINCT&nbsp;} (no parentheses, one trailing space), and fixes
+   *
+   * @see <a
+   * href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=402600">EclipseLink
+   * bug 402600</a>
+   */
+  protected ExpressionOperator distinctOperator() {
+    ExpressionOperator operator = ExpressionOperator.distinct();
+    operator.printsAs("DISTINCT "); // no parens, one space
+    return operator;
+  }
+
   /**
    * Overrides the default behavior to return {@code false},
    * indicating that ANSI {@code OUTER JOIN} syntax should be used,
@@ -342,7 +355,7 @@ public class Informix11Platform extends InformixPlatform {
     }
     return new DatabaseTable("TL_" + table.getName(), "" /* no table qualifier */, table.shouldUseDelimiters(), this.getStartDelimiter(), this.getEndDelimiter());
   }
-  
+
   /**
    * Returns {@code WITH NO LOG} when invoked, per the <a
    * href="http://publib.boulder.ibm.com/infocenter/idshelp/v117/topic/com.ibm.sqls.doc/ids_sqs_0571.htm">Informix
@@ -399,18 +412,18 @@ public class Informix11Platform extends InformixPlatform {
   @Override
   public void writeUpdateOriginalFromTempTableSql(final Writer writer, final DatabaseTable table, final Collection pkFields, final Collection assignedFields) throws IOException {
     writer.write("UPDATE ");
-    final String tableName = table.getQualifiedNameDelimited(this);    
+    final String tableName = table.getQualifiedNameDelimited(this);
     writer.write(tableName);
     writer.write(" SET ");
     final int size = assignedFields.size();
     if (size > 1) {
-      writer.write("(");            
+      writer.write("(");
     }
     writeFieldsList(writer, assignedFields, this);
     if (size > 1) {
-      writer.write(")");            
+      writer.write(")");
     }
-    writer.write(" = (SELECT ");        
+    writer.write(" = (SELECT ");
     writeFieldsList(writer, assignedFields, this);
     writer.write(" FROM ");
     final String tempTableName = this.getTempTableForTable(table).getQualifiedNameDelimited(this);
@@ -473,8 +486,6 @@ public class Informix11Platform extends InformixPlatform {
     case Types.LONGVARCHAR:
       if (resultSet != null) {
         returnValue = resultSet.getString(columnNumber);
-      } else {
-        returnValue = super.getObjectFromResultSet(resultSet, columnNumber, jdbcType, session);
       }
       break;
     default:
