@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1998, 2019 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 1998, 2019 IBM Corporation. All rights reserved.
+ * Copyright (c) 1998, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022 IBM Corporation. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v. 2.0 which is available at
@@ -236,14 +236,6 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         writer.write("'");
     }
 
-    /**
-     * INTERNAL:
-     * Build operator.
-     */
-    public ExpressionOperator atan2Operator() {
-        return ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Atan2, "ATN2");
-    }
-
     @Override
     protected Hashtable buildFieldTypes() {
         Hashtable fieldTypeMapping;
@@ -463,8 +455,8 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.CurrentDate, "GETDATE"));
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.CurrentTime, "GETDATE"));
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.Length, "CHAR_LENGTH"));
-        addOperator(ExpressionOperator.sybaseLocateOperator());
-        addOperator(ExpressionOperator.sybaseLocate2Operator());
+        addOperator(sybaseLocateOperator());
+        addOperator(sybaseLocate2Operator());
         addOperator(ExpressionOperator.simpleThreeArgumentFunction(ExpressionOperator.Substring, "SUBSTRING"));
         addOperator(singleArgumentSubstringOperator());
         addOperator(ExpressionOperator.addDate());
@@ -478,16 +470,16 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         addOperator(ExpressionOperator.replicate());
         addOperator(ExpressionOperator.right());
         addOperator(ExpressionOperator.cot());
-        addOperator(ExpressionOperator.sybaseAtan2Operator());
-        addOperator(ExpressionOperator.sybaseAddMonthsOperator());
-        addOperator(ExpressionOperator.sybaseInStringOperator());
+        addOperator(ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Atan2, "ATN2"));
+        addOperator(sybaseAddMonthsOperator());
+        addOperator(sybaseInStringOperator());
         // bug 3061144
         addOperator(ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Nvl, "ISNULL"));
         // CR### TO_NUMBER, TO_CHAR, TO_DATE is CONVERT(type, ?)
-        addOperator(ExpressionOperator.sybaseToNumberOperator());
-        addOperator(ExpressionOperator.sybaseToDateToStringOperator());
-        addOperator(ExpressionOperator.sybaseToDateOperator());
-        addOperator(ExpressionOperator.sybaseToCharOperator());
+        addOperator(sybaseToNumberOperator());
+        addOperator(sybaseToDateToStringOperator());
+        addOperator(sybaseToDateOperator());
+        addOperator(sybaseToCharOperator());
         addOperator(ExpressionOperator.simpleFunction(ExpressionOperator.Ceil, "CEILING"));
         addOperator(modOperator());
         addOperator(trimOperator());
@@ -497,11 +489,240 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         addOperator(extractOperator());
     }
 
+    /**
+     * Override the default MOD operator.
+     */
+    protected static ExpressionOperator modOperator() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.Mod);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
+        v.addElement(" % ");
+        result.printsAs(v);
+        result.bePostfix();
+        result.setNodeClass(org.eclipse.persistence.internal.expressions.FunctionExpression.class);
+        return result;
+    }
+
+    /**
+     *  Create the outer join operator for this platform
+     */
+    protected static ExpressionOperator operatorOuterJoin() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.EqualOuterJoin);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
+        v.addElement(" =* ");
+        result.printsAs(v);
+        result.bePostfix();
+        result.setNodeClass(RelationExpression.class);
+        return result;
+    }
+
+    /**
+     * Override the default SubstringSingleArg operator.
+     */
+    protected static ExpressionOperator singleArgumentSubstringOperator() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.SubstringSingleArg);
+        result.setType(ExpressionOperator.FunctionOperator);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
+        v.addElement("SUBSTRING(");
+        v.addElement(",");
+        v.addElement(", CHAR_LENGTH(");
+        v.addElement("))");
+        result.printsAs(v);
+        int[] indices = new int[3];
+        indices[0] = 0;
+        indices[1] = 1;
+        indices[2] = 0;
+
+        result.setArgumentIndices(indices);
+        result.setNodeClass(ClassConstants.FunctionExpression_Class);
+        result.bePrefix();
+        return result;
+    }
+
+    /**
+     * INTERNAL:
+     * Function, to add months to a date.
+     */
+    protected static ExpressionOperator sybaseAddMonthsOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.AddMonths);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(3);
+        v.add("DATEADD(month, ");
+        v.add(", ");
+        v.add(")");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        int[] indices = { 1, 0 };
+        exOperator.setArgumentIndices(indices);
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Build instring operator
+     */
+    protected static ExpressionOperator sybaseInStringOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.Instring);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(3);
+        v.add("CHARINDEX(");
+        v.add(", ");
+        v.add(")");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        int[] indices = { 1, 0 };
+        exOperator.setArgumentIndices(indices);
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Build Sybase equivalent to TO_NUMBER.
+     */
+    protected static ExpressionOperator sybaseToNumberOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.ToNumber);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(2);
+        v.add("CONVERT(NUMERIC, ");
+        v.add(")");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Build Sybase equivalent to TO_CHAR.
+     */
+    protected static ExpressionOperator sybaseToDateToStringOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.DateToString);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(2);
+        v.add("CONVERT(CHAR, ");
+        v.add(")");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Build Sybase equivalent to TO_DATE.
+     */
+    protected static ExpressionOperator sybaseToDateOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.ToDate);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(2);
+        v.add("CONVERT(DATETIME, ");
+        v.add(")");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Build Sybase equivalent to TO_CHAR.
+     */
+    protected static ExpressionOperator sybaseToCharOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.ToChar);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(2);
+        v.add("CONVERT(CHAR, ");
+        v.add(")");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Build Sybase equivalent to TO_CHAR.
+     */
+    protected static ExpressionOperator sybaseToCharWithFormatOperator() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.ToCharWithFormat);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(3);
+        v.add("CONVERT(CHAR, ");
+        v.add(",");
+        v.add(")");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
+    /**
+     * INTERNAL:
+     * Build the Sybase equivalent to Locate
+     */
+    protected static ExpressionOperator sybaseLocateOperator() {
+        ExpressionOperator result = ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Locate, "CHARINDEX");
+        int[] argumentIndices = new int[2];
+        argumentIndices[0] = 1;
+        argumentIndices[1] = 0;
+        result.setArgumentIndices(argumentIndices);
+        return result;
+    }
+
+    /**
+     * INTERNAL:
+     * Build the Sybase equivalent to Locate with a start index.
+     * Sybase does not define this, so this gets a little complex...
+     */
+    protected static ExpressionOperator sybaseLocate2Operator() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.Locate2);
+        result.setType(ExpressionOperator.FunctionOperator);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
+        v.add("CASE (CHARINDEX(");
+        v.add(", SUBSTRING(");
+        v.add(",");
+        v.add(", CHAR_LENGTH(");
+        v.add(")))) WHEN 0 THEN 0 ELSE (CHARINDEX(");
+        v.add(", SUBSTRING(");
+        v.add(",");
+        v.add(", CHAR_LENGTH(");
+        v.add("))) + ");
+        v.add(" - 1) END");
+        result.printsAs(v);
+        int[] indices = new int[9];
+        indices[0] = 1;
+        indices[1] = 0;
+        indices[2] = 2;
+        indices[3] = 0;
+        indices[4] = 1;
+        indices[5] = 0;
+        indices[6] = 2;
+        indices[7] = 0;
+        indices[8] = 2;
+
+        result.setArgumentIndices(indices);
+        result.setNodeClass(ClassConstants.FunctionExpression_Class);
+        result.bePrefix();
+        return result;
+    }
+
     /**
      * INTERNAL:
      * Derby does not support EXTRACT, but does have DATEPART.
      */
-    public static ExpressionOperator extractOperator() {
+    protected static ExpressionOperator extractOperator() {
         ExpressionOperator exOperator = new ExpressionOperator();
         exOperator.setType(ExpressionOperator.FunctionOperator);
         exOperator.setSelector(ExpressionOperator.Extract);
@@ -524,7 +745,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Use RTRIM(LTRIM(?)) function for trim.
      */
-    public static ExpressionOperator trimOperator() {
+    protected static ExpressionOperator trimOperator() {
         ExpressionOperator exOperator = new ExpressionOperator();
         exOperator.setType(ExpressionOperator.FunctionOperator);
         exOperator.setSelector(ExpressionOperator.Trim);
@@ -541,7 +762,7 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Build Trim operator.
      */
-    public static ExpressionOperator trim2Operator() {
+    protected static ExpressionOperator trim2Operator() {
         ExpressionOperator exOperator = new ExpressionOperator();
         exOperator.setType(ExpressionOperator.FunctionOperator);
         exOperator.setSelector(ExpressionOperator.Trim2);
@@ -608,34 +829,6 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         return values;
     }
 
-    /**
-     * Override the default MOD operator.
-     */
-    public ExpressionOperator modOperator() {
-        ExpressionOperator result = new ExpressionOperator();
-        result.setSelector(ExpressionOperator.Mod);
-        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
-        v.addElement(" % ");
-        result.printsAs(v);
-        result.bePostfix();
-        result.setNodeClass(org.eclipse.persistence.internal.expressions.FunctionExpression.class);
-        return result;
-    }
-
-    /*
-     *  Create the outer join operator for this platform
-     */
-    protected ExpressionOperator operatorOuterJoin() {
-        ExpressionOperator result = new ExpressionOperator();
-        result.setSelector(ExpressionOperator.EqualOuterJoin);
-        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
-        v.addElement(" =* ");
-        result.printsAs(v);
-        result.bePostfix();
-        result.setNodeClass(RelationExpression.class);
-        return result;
-    }
-
     /** Append the receiver's field 'identity' constraint clause to a writer.*/
     @Override
     public void printFieldIdentityClause(Writer writer) throws ValidationException {
@@ -721,30 +914,6 @@ public class SybasePlatform extends org.eclipse.persistence.platform.database.Da
         return false;
     }
 
-    /**
-     * Override the default SubstringSingleArg operator.
-     */
-    public ExpressionOperator singleArgumentSubstringOperator() {
-        ExpressionOperator result = new ExpressionOperator();
-        result.setSelector(ExpressionOperator.SubstringSingleArg);
-        result.setType(ExpressionOperator.FunctionOperator);
-        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance();
-        v.addElement("SUBSTRING(");
-        v.addElement(",");
-        v.addElement(", CHAR_LENGTH(");
-        v.addElement("))");
-        result.printsAs(v);
-        int[] indices = new int[3];
-        indices[0] = 0;
-        indices[1] = 1;
-        indices[2] = 0;
-
-        result.setArgumentIndices(indices);
-        result.setNodeClass(ClassConstants.FunctionExpression_Class);
-        result.bePrefix();
-        return result;
-    }
-
     /**
      * INTERNAL:
      * Indicates whether the platform supports identity.
