@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2023 Oracle and/or its affiliates. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v. 2.0 which is available at
@@ -210,7 +210,7 @@ public class DMSPerformanceProfiler implements Serializable, Cloneable, SessionP
         }
         Sensor phaseEvent = getSensorByName(operationName);
         if (phaseEvent != null) {
-            Long startToken = new Long(((PhaseEvent)phaseEvent).start());
+            Long startToken = Long.valueOf(((PhaseEvent)phaseEvent).start());
             getPhaseEventStartToken().put(operationName, startToken);
         }
     }
@@ -231,7 +231,7 @@ public class DMSPerformanceProfiler implements Serializable, Cloneable, SessionP
 
         Sensor phaseEvent = getPhaseEventForQuery(operationName, query, weight);
         if (phaseEvent != null) {
-            Long startToken = new Long(((PhaseEvent)phaseEvent).start());
+            Long startToken = Long.valueOf(((PhaseEvent)phaseEvent).start());
             if (query != null) {
                 getPhaseEventStartToken().put(query.getSensorName(operationName, getSessionName()), startToken);
             } else {
