@@ -1,16 +1,19 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 1998, 2019 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019 IBM Corporation. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- *     Markus KARG - Added methods allowing to support stored procedure creation on SQLAnywherePlatform. 
- ******************************************************************************/  
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     Markus KARG - Added methods allowing to support stored procedure creation on SQLAnywherePlatform.
 package org.eclipse.persistence.tools.schemaframework;
 
 import java.io.IOException;
@@ -169,12 +172,15 @@ public class StoredProcedureDefinition extends DatabaseObjectDefinition {
             printReturn(writer, session);
             writer.write(platform.getProcedureAsString());
             writer.write("\n");
-            
+
+            writer.write(platform.getProcedureOptionList());
+            writer.write("\n");
+
             if (platform.shouldPrintStoredProcedureVariablesAfterBeginString()) {
                 writer.write(platform.getProcedureBeginString());
                 writer.write("\n");
             }
-            
+
             if (!getVariables().isEmpty()) {
                 writer.write("DECLARE\n");
             }
@@ -189,7 +195,7 @@ public class StoredProcedureDefinition extends DatabaseObjectDefinition {
                 writer.write(platform.getBatchDelimiterString());
                 writer.write("\n");
             }
-            
+
             if (!platform.shouldPrintStoredProcedureVariablesAfterBeginString()) {
                 writer.write(platform.getProcedureBeginString());
                 writer.write("\n");
